import re
import warnings

from functools import reduce

from urllib.parse import urlparse

from collections import OrderedDict

from django.conf import settings as django_settings
from django.http.response import HttpResponse, HttpResponseBase
from django.utils.decorators import classonlymethod
from django.utils.encoding import force_text
from django.db.models.base import Model
from django.db.models.query import QuerySet
from django.db.models.fields import DateTimeField
from django.http.response import Http404
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _
from django.utils.module_loading import import_string

from functools import update_wrapper

from chamber.shortcuts import get_object_or_none
from chamber.exceptions import PersistenceException
from chamber.utils import remove_accent, transaction

from .conf import settings
from .paginator import BaseOffsetPaginator
from .response import (HeadersResponse, RESTCreatedResponse, RESTNoContentResponse, ResponseErrorFactory,
                       ResponseExceptionFactory)
from .exception import (RESTException, ConflictException, NotAllowedException, DataInvalidException,
                        ResourceNotFoundException, NotAllowedMethodException, DuplicateEntryException,
                        UnsupportedMediaTypeException, MimerDataException, UnauthorizedException)
from .forms import ISODateTimeField, RESTModelForm, rest_modelform_factory
from .utils import coerce_rest_request_method, set_rest_context_to_request, RFS, rfs
from .utils.helpers import str_to_class
from .serializer import (
    ResourceSerializer, ModelResourceSerializer, LazyMappedSerializedData, ObjectResourceSerializer
)
from .converters import get_converter_name_from_request, get_converter_from_request
from .filters.managers import MultipleFilterManager
from .order.managers import DefaultModelOrderManager


ACCESS_CONTROL_ALLOW_ORIGIN = 'Access-Control-Allow-Origin'
ACCESS_CONTROL_EXPOSE_HEADERS = 'Access-Control-Expose-Headers'
ACCESS_CONTROL_ALLOW_CREDENTIALS = 'Access-Control-Allow-Credentials'
ACCESS_CONTROL_ALLOW_HEADERS = 'Access-Control-Allow-Headers'
ACCESS_CONTROL_ALLOW_METHODS = 'Access-Control-Allow-Methods'
ACCESS_CONTROL_MAX_AGE = 'Access-Control-Max-Age'


typemapper = {}
resource_tracker = []


class ResourceMetaClass(type):
    """
    Metaclass that keeps a registry of class -> resource
    mappings.
    """
    def __new__(cls, name, bases, attrs):
        abstract = attrs.pop('abstract', False)
        new_cls = type.__new__(cls, name, bases, attrs)
        if not abstract and new_cls.register and settings.AUTO_REGISTER_RESOURCE:
            def already_registered(model):
                return typemapper.get(model)

            if hasattr(new_cls, 'model'):
                if already_registered(new_cls.model) and not settings.IGNORE_DUPE_MODELS:
                    warnings.warn('Resource already registered for model {}, '
                                  'you may experience inconsistent results.'.format(new_cls.model.__name__))

                typemapper[new_cls.model] = new_cls

            if name != 'BaseResource':
                resource_tracker.append(new_cls)

        if not abstract:
            converters = OrderedDict()
            for converter_class_path in new_cls.converter_classes:
                converter_class = (
                    import_string(converter_class_path) if isinstance(converter_class_path, str)
                    else converter_class_path
                )
                converters[converter_class.format] = converter_class()
            new_cls.converters = converters
        return new_cls


class PermissionsResourceMixin:

    allowed_methods = ('get', 'post', 'put', 'patch', 'delete', 'head', 'options')

    def get_allowed_methods(self):
        return set(self.allowed_methods)

    def _get_via(self, via=None):
        via = list(via) if via is not None else []
        via.append(self)
        return via

    def check_permissions_and_get_allowed_methods(self, restricted_methods=None, **kwargs):
        allowed_methods = []

        tested_methods = self.get_allowed_methods()
        if restricted_methods is not None:
            tested_methods = tested_methods.intersection(restricted_methods)

        for method in tested_methods:
            try:
                self._check_permission(method, **kwargs)
                allowed_methods.append(method)
            except (NotImplementedError, NotAllowedException, UnauthorizedException):
                pass
        return allowed_methods

    def _check_permission(self, name=None, *args, **kwargs):
        name = name or self.request.method.lower()

        if not hasattr(self, 'has_{}_permission'.format(name)):
            if django_settings.DEBUG:
                raise NotImplementedError('Please implement method has_{}_permission to {}'.format(name, self.__class__))
            else:
                raise NotAllowedException

        if not getattr(self, 'has_{}_permission'.format(name))(*args, **kwargs):
            raise NotAllowedException

    def has_permission(self, name=None, *args, **kwargs):
        name = name or self.request.method.lower()

        if not hasattr(self, 'has_{}_permission'.format(name)):
            if django_settings.DEBUG:
                raise NotImplementedError('Please implement method has_{}_permission to {}'.format(name, self.__class__))
            else:
                return False
        try:
            return getattr(self, 'has_{}_permission'.format(name))(*args, **kwargs)
        except Http404:
            return False

    def has_get_permission(self, **kwargs):
        return 'get' in self.get_allowed_methods() and hasattr(self, 'get')

    def has_post_permission(self, **kwargs):
        return 'post' in self.get_allowed_methods() and hasattr(self, 'post')

    def has_put_permission(self, **kwargs):
        return 'put' in self.get_allowed_methods() and hasattr(self, 'put')

    def has_patch_permission(self, **kwargs):
        return 'patch' in self.get_allowed_methods() and hasattr(self, 'patch')

    def has_delete_permission(self, **kwargs):
        return 'delete' in self.get_allowed_methods() and hasattr(self, 'delete')

    def has_head_permission(self, **kwargs):
        return 'head' in self.get_allowed_methods() and (hasattr(self, 'head') or self.has_get_permission(**kwargs))

    def has_options_permission(self, **kwargs):
        return 'options' in self.get_allowed_methods() and hasattr(self, 'options')


class ObjectPermissionsResourceMixin(PermissionsResourceMixin):

    can_create_obj = False
    can_read_obj = False
    can_update_obj = False
    can_delete_obj = False

    def has_create_obj_permission(self, obj=None, via=None):
        return self.can_create_obj

    def has_read_obj_permission(self, obj=None, via=None):
        return self.can_read_obj

    def has_update_obj_permission(self, obj=None, via=None):
        return self.can_update_obj

    def has_delete_obj_permission(self, obj=None, via=None):
        return self.can_delete_obj


class BaseResource(PermissionsResourceMixin, metaclass=ResourceMetaClass):
    """
    BaseResource that gives you CRUD for free.
    You are supposed to subclass this for specific
    functionality.

    All CRUD methods (`read`/`update`/`create`/`delete`)
    receive a request as the first argument from the
    resource. Use this for checking `request.user`, etc.
    """

    allowed_methods = ('get', 'post', 'put', 'patch', 'delete', 'head', 'options')
    serializer = ResourceSerializer
    register = False
    abstract = True
    csrf_exempt = True
    cache = None
    paginator = BaseOffsetPaginator
    resource_typemapper = {}
    converter_classes = settings.CONVERTERS
    errors_response_class = settings.ERRORS_RESPONSE_CLASS
    error_response_class = settings.ERROR_RESPONSE_CLASS

    DEFAULT_REST_CONTEXT_MAPPING = {
        'serialization_format': ('HTTP_X_SERIALIZATION_FORMAT', '_serialization_format'),
        'fields': ('HTTP_X_FIELDS', '_fields'),
        'offset': ('HTTP_X_OFFSET', '_offset'),
        'base': ('HTTP_X_BASE', '_base'),
        'accept': ('HTTP_ACCEPT', '_accept'),
        'content_type': ('CONTENT_TYPE', '_content_type'),
        'filter': ('HTTP_X_FILTER', 'filter'),
        'order': ('HTTP_X_ORDER', 'order'),
    }
    DATA_KEY_MAPPING = {}

    def __init__(self, request):
        self.request = request
        self.args = []
        self.kwargs = {}

    def get_allowed_methods(self):
        allowed_methods = super().get_allowed_methods()
        if self.is_allowed_cors:
            allowed_methods.add('options')
        return allowed_methods

    @property
    def exception_responses(self):
        errors_response_class = (
            str_to_class(self.errors_response_class) if isinstance(self.errors_response_class, str)
            else self.errors_response_class
        )
        error_response_class = (
            str_to_class(self.error_response_class) if isinstance(self.error_response_class, str)
            else self.error_response_class
        )
        return (
            (MimerDataException, ResponseErrorFactory(_('Bad Request'), 400, error_response_class)),
            (UnauthorizedException, ResponseErrorFactory(_('Unauthorized'), 401, error_response_class)),
            (NotAllowedException, ResponseErrorFactory(_('Forbidden'), 403, error_response_class)),
            (UnsupportedMediaTypeException, ResponseErrorFactory(_('Unsupported Media Type'), 415,
                                                                 error_response_class)),
            (Http404, ResponseErrorFactory(_('Not Found'), 404, error_response_class)),
            (ResourceNotFoundException, ResponseErrorFactory(_('Not Found'), 404, error_response_class)),
            (NotAllowedMethodException, ResponseErrorFactory(_('Method Not Allowed'), 405, error_response_class)),
            (DuplicateEntryException, ResponseErrorFactory(_('Conflict/Duplicate'), 409, error_response_class)),
            (ConflictException, ResponseErrorFactory(_('Conflict/Duplicate'), 409, error_response_class)),
            (DataInvalidException, ResponseExceptionFactory(errors_response_class)),
            (RESTException, ResponseExceptionFactory(error_response_class)),
            (PersistenceException, ResponseExceptionFactory(error_response_class)),
        )

    @property
    def is_allowed_cors(self):
        return settings.CORS

    @property
    def cors_whitelist(self):
        return settings.CORS_WHITELIST

    @property
    def cors_max_age(self):
        return settings.CORS_MAX_AGE

    def _demap_key(self, lookup_key):
        return {v: k for k, v in self.DATA_KEY_MAPPING.items()}.get(lookup_key, lookup_key)

    def update_serialized_data(self, data):
        if data and self.DATA_KEY_MAPPING:
            data = LazyMappedSerializedData(data, self.DATA_KEY_MAPPING).serialize()
        return data

    def update_deserialized_data(self, data):
        return (
            {self._demap_key(k): v for k, v in data.items() if k not in self.DATA_KEY_MAPPING}
            if isinstance(data, dict) else {}
        )

    def get_dict_data(self):
        return self.update_deserialized_data(self.request.data if hasattr(self.request, 'data') else {})

    def _get_serialization_format(self):
        serialization_format = self.request._rest_context.get('serialization_format',
                                                              self.serializer.SERIALIZATION_TYPES.RAW)
        if serialization_format not in self.serializer.SERIALIZATION_TYPES:
            return self.serializer.SERIALIZATION_TYPES.RAW
        return serialization_format

    def head(self):
        return self.get()

    def _get_cors_allowed_headers(self):
        return settings.CORS_ALLOWED_HEADERS

    def _get_cors_allowed_exposed_headers(self):
        return settings.CORS_ALLOWED_EXPOSED_HEADERS

    def _cors_is_origin_allowed(self, origin):
        if not origin:
            return False
        elif self.cors_whitelist == '__all__':
            return True
        else:
            url = urlparse(origin)
            return url.netloc in self.cors_whitelist or self._regex_domain_match(origin)

    def _regex_domain_match(self, origin):
        for domain_pattern in self.cors_whitelist:
            if re.match(domain_pattern, origin):
                return origin

    def _is_cors_options_request(self):
        return (
            self.is_allowed_cors and self.request.method.upper() == 'OPTIONS' and self.request.META.get('HTTP_ORIGIN')
        )

    def options(self):
        if self._is_cors_options_request():
            http_headers = {
                ACCESS_CONTROL_ALLOW_METHODS: self.request.META.get('HTTP_ACCESS_CONTROL_REQUEST_METHOD', 'OPTIONS'),
                ACCESS_CONTROL_ALLOW_HEADERS: ', '.join(self._get_cors_allowed_headers())
            }
            return HeadersResponse(None, http_headers=http_headers)
        else:
            return None

    def _get_converted_dict(self, result):
        return self._get_converted_serialized_data(result)

    def _get_converted_serialized_data(self, result):
        return self.serializer(self, request=self.request).serialize(
            result, self._get_serialization_format(), lazy=True, allow_tags=self._get_converter().allow_tags
        )

    def _get_converter(self):
        try:
            return get_converter_from_request(self.request, self.converters)
        except ValueError:
            raise UnsupportedMediaTypeException

    def _serialize(self, os, result, status_code, http_headers):
        converter = self._get_converter()
        http_headers['Content-Type'] = converter.content_type

        converter.encode_to_stream(os, self._get_converted_dict(result), resource=self, request=self.request,
                                   status_code=status_code, http_headers=http_headers, result=result)

    def _deserialize(self):
        rm = self.request.method.upper()
        # Django's internal mechanism doesn't pick up
        # PUT request, so we trick it a little here.
        if rm in {'PUT', 'PATCH'}:
            coerce_rest_request_method(self.request)

        if rm in {'POST', 'PUT', 'PATCH'}:
            try:
                converter = get_converter_from_request(self.request, self.converters, True)
                self.request.data = self.serializer(self).deserialize(converter.decode(force_text(self.request.body)))
            except (TypeError, ValueError):
                raise MimerDataException
            except NotImplementedError:
                raise UnsupportedMediaTypeException
        return self.request

    def _get_error_response_from_exception(self, exception):
        for exception_class, response_factory in self.exception_responses:
            if isinstance(exception, exception_class):
                return response_factory.get_response(exception)

    def _get_response_data(self):
        status_code = 200
        http_headers = {}

        fieldset = True
        try:
            self.request = self._deserialize()

            rm = self.request.method.lower()
            meth = getattr(self, rm, None)

            if not meth or rm not in self.get_allowed_methods():
                raise NotAllowedMethodException
            else:
                self._check_permission(rm)
                result = meth()
        except Exception as ex:
            result = self._get_error_response_from_exception(ex)
            if result is None:
                raise ex
            fieldset = False

        if isinstance(result, HeadersResponse):
            fieldset = result.fieldset
            http_headers = result.http_headers
            status_code = result.status_code
            result = result.result

        if isinstance(result, HttpResponse):
            status_code = result.status_code
            result = result._container
        return result, http_headers, status_code, fieldset

    def _set_response_headers(self, response, http_headers):
        for header, value in http_headers.items():
            response[header] = value

    def _get_from_cache(self):
        if self.cache:
            return self.cache.get_response(self.request)

    def _store_to_cache(self, response):
        if self.cache and response.status_code < 400:
            self.cache.cache_response(self.request, response)

    def _get_headers_queryset_context_mapping(self):
        return self.DEFAULT_REST_CONTEXT_MAPPING.copy()

    def _get_context(self):
        context = {}
        for key, (header_key, queryset_key) in self._get_headers_queryset_context_mapping().items():
            val = self.request.GET.get(queryset_key, self.request.META.get(header_key))
            if val:
                context[key] = val
        return context

    def render_response(self, result, http_headers, status_code, fieldset):
        if isinstance(result, HttpResponseBase):
            return result
        else:
            if not fieldset and 'fields' in self.request._rest_context:
                del self.request._rest_context['fields']
            response = HttpResponse()
            try:
                response.status_code = status_code
                http_headers = self._get_headers(http_headers)
                self._serialize(response, result, status_code, http_headers)
            except UnsupportedMediaTypeException:
                response.status_code = 415
                http_headers['Content-Type'] = self.request.get('HTTP_ACCEPT')

            self._set_response_headers(response, http_headers)
            return response

    def dispatch(self, request, *args, **kwargs):
        set_rest_context_to_request(request, self._get_headers_queryset_context_mapping())
        response = self._get_from_cache()
        if response:
            return response
        else:
            response = self.render_response(*self._get_response_data())
            self._store_to_cache(response)
            return response

    def _get_name(self):
        return 'resource'

    def _get_filename(self):
        return '{}.{}'.format(self._get_name(), get_converter_name_from_request(self.request, self.converters))

    def _get_allow_header(self):
        return ','.join((method.upper() for method in self.check_permissions_and_get_allowed_methods()))

    def _get_headers(self, default_http_headers):
        origin = self.request.META.get('HTTP_ORIGIN')

        http_headers = default_http_headers.copy()
        http_headers['Cache-Control'] = 'private, no-cache, no-store, max-age=0'
        http_headers['Pragma'] = 'no-cache'
        http_headers['Expires'] = '0'
        http_headers['Vary'] = 'Accept'

        if self.has_permission():
            http_headers['X-Serialization-Format-Options'] = ','.join(self.serializer.SERIALIZATION_TYPES)
            http_headers['Content-Disposition'] = 'inline; filename="{}"'.format(self._get_filename())
            http_headers['Allow'] = self._get_allow_header()

        if self.is_allowed_cors:
            if origin and self._cors_is_origin_allowed(origin):
                http_headers[ACCESS_CONTROL_ALLOW_ORIGIN] = origin
            http_headers[ACCESS_CONTROL_ALLOW_CREDENTIALS] = (
                'true' if settings.CORS_ALLOW_CREDENTIALS else 'false'
            )
            cors_allowed_exposed_headers = self._get_cors_allowed_exposed_headers()
            if cors_allowed_exposed_headers:
                http_headers[ACCESS_CONTROL_EXPOSE_HEADERS] = ', '.join(cors_allowed_exposed_headers)
            http_headers[ACCESS_CONTROL_MAX_AGE] = str(self.cors_max_age)
        return http_headers

    @classonlymethod
    def as_view(cls, allowed_methods=None, **initkwargs):
        def view(request, *args, **kwargs):
            self = cls(request, **initkwargs)
            self.request = request
            self.args = args
            self.kwargs = kwargs
            if allowed_methods is not None:
                self.allowed_methods = set(allowed_methods) & set(cls.allowed_methods)
            else:
                self.allowed_methods = set(cls.allowed_methods)

            return self.dispatch(request, *args, **kwargs)
        view.csrf_exempt = cls.csrf_exempt

        # take name and docstring from class
        update_wrapper(view, cls, updated=())

        # and possible attributes set by decorators
        # like csrf_exempt from dispatch
        update_wrapper(view, cls.dispatch, assigned=())
        return view


def join_rfs(*iterable):
    return reduce(lambda a, b: a.join(b), iterable, rfs())


class DefaultRESTObjectResource(ObjectPermissionsResourceMixin):

    fields = None
    allowed_fields = None
    detailed_fields = None
    general_fields = None
    guest_fields = None
    allowed_methods = None
    default_fields = None
    extra_fields = None
    filter_fields = None
    extra_filter_fields = None
    order_fields = None
    extra_order_fields = None

    def get_allowed_fields_rfs(self, obj=None):
        return rfs(self.allowed_fields) if self.allowed_fields is not None else join_rfs(
            self.get_fields_rfs(obj),
            self.get_detailed_fields_rfs(obj),
            self.get_general_fields_rfs(obj),
            self.get_extra_fields_rfs(obj),
            self.get_default_fields_rfs(obj)
        )

    def get_fields(self, obj=None):
        return list(self.fields) if self.fields is not None else None

    def get_default_fields(self, obj=None):
        return list(self.default_fields) if self.default_fields is not None else None

    def get_detailed_fields(self, obj=None):
        return list(self.detailed_fields) if self.detailed_fields is not None else self.get_fields(obj=obj)

    def get_general_fields(self, obj=None):
        return list(self.general_fields) if self.general_fields is not None else self.get_fields(obj=obj)

    def get_guest_fields(self, obj=None):
        return list(self.guest_fields) if self.guest_fields is not None else None

    def get_extra_fields(self, obj=None):
        return list(self.extra_fields) if self.extra_fields is not None else None

    def get_fields_rfs(self, obj=None):
        fields = self.get_fields(obj=obj)

        return rfs(fields) if fields is not None else rfs()

    def get_default_fields_rfs(self, obj=None):
        default_fields = self.get_default_fields(obj=obj)

        return rfs(default_fields) if default_fields is not None else rfs()

    def get_detailed_fields_rfs(self, obj=None):
        detailed_fields = self.get_detailed_fields(obj=obj)

        return (rfs(detailed_fields) if detailed_fields is not None else rfs()).join(self.get_default_fields_rfs())

    def get_general_fields_rfs(self, obj=None):
        general_fields = self.get_general_fields(obj=obj)

        return (rfs(general_fields) if general_fields is not None else rfs()).join(self.get_default_fields_rfs())

    def get_guest_fields_rfs(self, obj=None):
        guest_fields = self.get_guest_fields(obj=obj)

        return rfs(guest_fields) if guest_fields is not None else rfs()

    def get_extra_fields_rfs(self, obj=None):
        extra_fields = self.get_extra_fields(obj=obj)

        return rfs(extra_fields) if extra_fields is not None else rfs()

    def get_extra_filter_fields(self):
        """
        :return: filter fields list that excludes default filter fields.
        """
        return list(self.extra_filter_fields) if self.extra_filter_fields is not None else None

    def get_filter_fields(self):
        """
        :return: filter fields list or None.
        """
        return list(self.filter_fields) if self.filter_fields is not None else None

    def get_filter_fields_rfs(self):
        """
        :return: RFS of allowed filter fields. If filter_fields is None value is returned from all allowed fields to
        read.
        """
        filter_fields = self.get_filter_fields()
        extra_filter_fields = self.get_extra_filter_fields() or ()
        if filter_fields is None:
            return rfs(extra_filter_fields).join(self.get_allowed_fields_rfs())
        else:
            return rfs(extra_filter_fields).join(rfs(filter_fields))

    def get_extra_order_fields(self):
        """
        :return: order fields list that excludes default filter fields.
        """
        return list(self.extra_order_fields) if self.extra_order_fields is not None else None

    def get_order_fields(self):
        """
        :return: order fields list or None.
        """
        return list(self.order_fields) if self.order_fields is not None else None

    def get_order_fields_rfs(self):
        """
        :return: RFS of allowed order fields. If order_fields is None value is returned from all allowed fields to
        read.
        """
        order_fields = self.get_order_fields()
        extra_order_fields = self.get_extra_order_fields() or ()
        if order_fields is None:
            return rfs(extra_order_fields).join(self.get_allowed_fields_rfs())
        else:
            return rfs(extra_order_fields).join(rfs(order_fields))

    def get_methods_returning_field_value(self, fields):
        """
        Returns dict of resource methods which can be used with serializer to get a field value.
        :param fields: list of field names
        :return: dict of resource methods. Key is a field name, value is a method that returns field value.
        """
        method_fields = {}
        for field_name in fields:
            method = self.get_method_returning_field_value(field_name)
            if method:
                method_fields[field_name] = method
        return method_fields

    def get_method_returning_field_value(self, field_name):
        """
        Returns method which can be used with serializer to get a field value.
        :param field_name: name of th field
        :return: resource method
        """
        method = getattr(self, field_name, None)
        return method if method and callable(method) else None


class DefaultRESTModelResource(DefaultRESTObjectResource):

    allowed_methods = ('get', 'post', 'put', 'patch', 'delete', 'head', 'options')
    model = None

    def get_detailed_fields(self, obj=None):
        detailed_fields = super().get_detailed_fields(obj=obj)
        return list(self.model._rest_meta.detailed_fields) if detailed_fields is None else detailed_fields

    def get_general_fields(self, obj=None):
        general_fields = super().get_general_fields(obj=obj)
        return list(self.model._rest_meta.general_fields) if general_fields is None else general_fields

    def get_guest_fields(self, obj=None):
        guest_fields = super().get_guest_fields(obj=obj)
        return list(self.model._rest_meta.guest_fields) if guest_fields is None else guest_fields

    def get_extra_fields(self, obj=None):
        extra_fields = super().get_extra_fields(obj=obj)
        return list(self.model._rest_meta.extra_fields) if extra_fields is None else extra_fields

    def get_default_fields(self, obj=None):
        default_fields = super().get_default_fields(obj=obj)
        return list(self.model._rest_meta.default_fields) if default_fields is None else default_fields

    def get_extra_filter_fields(self):
        extra_filter_fields = super().get_extra_filter_fields()
        return list(self.model._rest_meta.extra_filter_fields) if extra_filter_fields is None else extra_filter_fields

    def get_filter_fields(self):
        filter_fields = super().get_filter_fields()
        return self.model._rest_meta.filter_fields if filter_fields is None else filter_fields

    def get_extra_order_fields(self):
        extra_order_fields = super().get_extra_order_fields()
        return list(self.model._rest_meta.extra_order_fields) if extra_order_fields is None else extra_order_fields

    def get_order_fields(self):
        order_fields = super().get_order_fields()
        return self.model._rest_meta.order_fields if order_fields is None else order_fields


class BaseObjectResource(DefaultRESTObjectResource, BaseResource):

    allowed_methods = ('get', 'post', 'put', 'patch', 'delete', 'head', 'options')
    pk_name = 'pk'
    pk_field_name = 'id'
    abstract = True
    partial_put_update = None
    partial_related_update = None
    serializer = ObjectResourceSerializer

    def _serialize(self, os, result, status_code, http_headers):
        try:
            converter = get_converter_from_request(self.request, self.converters)
            http_headers['Content-Type'] = converter.content_type

            converter.encode_to_stream(os, self._get_converted_dict(result), resource=self, request=self.request,
                                       status_code=status_code, http_headers=http_headers, result=result,
                                       requested_fields=self._get_requested_fieldset(result))
        except ValueError:
            raise UnsupportedMediaTypeException

    def _get_converted_serialized_data(self, result):
        return self.serializer(self, request=self.request).serialize(
            result, self._get_serialization_format(), requested_fieldset=self._get_requested_fieldset(result),
            lazy=True, allow_tags=self._get_converter().allow_tags
        )

    def _get_requested_fieldset(self, result):
        requested_fields = self.request._rest_context.get('fields')
        if requested_fields:
            return RFS.create_from_string(requested_fields)
        elif isinstance(result, Model):
            return self.get_detailed_fields_rfs(obj=result)
        elif isinstance(result, QuerySet):
            return self.get_general_fields_rfs()
        else:
            return None

    def _get_obj_or_404(self, pk=None):
        obj = self._get_obj_or_none(pk)
        if not obj:
            raise Http404
        return obj

    def render_response(self, result, http_headers, status_code, fieldset):
        return super(BaseObjectResource, self).render_response(result, http_headers, status_code, fieldset)

    def _get_allowed_fields_options_header(self):
        return ','.join(self.get_allowed_fields_rfs(self._get_obj_or_none()).flat())

    def _get_allow_header(self):
        return ','.join((
            method.upper() for method in self.check_permissions_and_get_allowed_methods(obj=self._get_obj_or_none())
        ))

    def _get_headers(self, default_http_headers):
        http_headers = super(BaseObjectResource, self)._get_headers(default_http_headers)
        if self.has_permission():
            http_headers['X-Fields-Options'] = self._get_allowed_fields_options_header()
        return http_headers

    def _get_queryset(self):
        """
        Should return list or db queryset
        """
        raise NotImplementedError

    def _get_obj_or_none(self, pk=None):
        """
        Should return one object
        """
        raise NotImplementedError

    def _filter_queryset(self, qs):
        """
        Should contain implementation for objects filtering
        """
        return qs

    def _preload_queryset(self, qs):
        """
        May contain preloading implementation for queryset
        """
        return qs

    def _order_queryset(self, qs):
        """
        Should contain implementation for objects ordering
        """
        return qs

    def _exists_obj(self, **kwargs):
        """
        Should return true if object exists
        """
        raise NotImplementedError

    def _get_pk(self):
        return self.kwargs.get(self.pk_name)

    def post(self):
        pk = self._get_pk()
        data = self.get_dict_data()
        if pk and self._exists_obj(pk=pk):
            raise DuplicateEntryException
        return RESTCreatedResponse(self.atomic_create_or_update(data))

    def get(self):
        pk = self._get_pk()
        if pk:
            return self._get_obj_or_404(pk=pk)
        qs = self._preload_queryset(self._get_queryset().all())
        qs = self._filter_queryset(qs)
        qs = self._order_queryset(qs)
        paginator = self.paginator(qs, self.request)
        return HeadersResponse(paginator.page_qs, paginator.headers)

    def put(self):
        pk = self._get_pk()
        data = self.get_dict_data()
        obj = self._get_obj_or_404(pk=pk)
        data[self.pk_field_name] = obj.pk
        try:
            # Backward compatibility
            partial_update = settings.PARTIAL_PUT_UPDATE if self.partial_put_update is None else self.partial_put_update
            return self.atomic_create_or_update(data, partial_update=partial_update)
        except ConflictException:
            # If object allready exists and user doesn't have permissions to change it should be returned 404 (the same
            # response as for GET method)
            raise Http404

    def patch(self):
        pk = self._get_pk()
        data = self.get_dict_data()
        obj = self._get_obj_or_404(pk=pk)
        data[self.pk_field_name] = obj.pk
        try:
            return self.atomic_create_or_update(data, partial_update=True)
        except ConflictException:
            # If object allready exists and user doesn't have permissions to change it should be returned 404 (the same
            # response as for GET method)
            raise Http404

    def delete(self):
        pk = self.kwargs.get(self.pk_name)
        self.delete_obj_with_pk(pk)
        return RESTNoContentResponse()

    def delete_obj_with_pk(self, pk, via=None):
        via = via or []
        obj = self._get_obj_or_404(pk)
        self._check_permission('delete_obj', obj=obj, via=via)
        self._pre_delete_obj(obj)
        self._delete_obj(obj)
        self._post_delete_obj(obj)

    def _pre_delete_obj(self, obj):
        pass

    def _delete_obj(self, obj):
        raise NotImplementedError

    def _post_delete_obj(self, obj):
        pass

    @transaction.atomic_with_signals
    def atomic_create_or_update(self, data, partial_update=False):
        """
        Atomic object creation
        """
        return self.create_or_update(data, partial_update=partial_update)

    def _get_instance(self, data):
        """
        Should contains implementation for get object according to input data values
        """
        raise NotImplementedError

    def _generate_form_class(self, inst, exclude=None):
        return self.form_class

    def _get_form(self, fields=None, inst=None, data=None, files=None, initial=None, partial_update=False):
        # When is send PUT (resource instance exists), it is possible send only changed values.
        initial = {} if initial is None else initial
        exclude = []

        kwargs = self._get_form_kwargs()
        if inst:
            kwargs['instance'] = inst
        if data is not None:
            kwargs['data'] = data
            kwargs['files'] = files

        form_class = self._generate_form_class(inst, exclude)
        return form_class(initial=initial, partial_update=partial_update, **kwargs)

    def _get_form_kwargs(self):
        return {}

    def _get_form_initial(self, obj):
        return {}

    def _can_save_obj(self, change, obj, form, via):
        if change and (not via or form.has_changed()):
            self._check_permission('update_obj', obj=obj, via=via)
        elif not change:
            self._check_permission('create_obj', obj=obj, via=via)

        return not change or self.has_update_obj_permission(obj=obj, via=via)

    def create_or_update(self, data, via=None, partial_update=False):
        try:
            return self._create_or_update(data, via, partial_update=partial_update)
        except DataInvalidException as ex:
            raise DataInvalidException(self._update_errors(ex.errors))

    def _update_errors(self, errors):
        return self.update_serialized_data(errors)

    def _create_or_update(self, data, via=None, partial_update=False):
        """
        Helper for creating or updating resource
        """
        from pyston.data_processor import data_preprocessors, data_postprocessors

        via = [] if via is None else via
        inst = self._get_instance(data)
        change = inst and True or False

        files = self.request.FILES.copy()

        form = self._get_form(inst=inst, data=data, initial=self._get_form_initial(inst))

        # Backward compatibility
        partial_related_update = (
            settings.PARTIAL_RELATED_UPDATE if self.partial_related_update is None else self.partial_related_update
        ) or partial_update

        for preprocessor in data_preprocessors.get_processors(type(self)):
            data, files = preprocessor(self, form, inst, via, partial_related_update).process_data(data, files)

        form = self._get_form(fields=form.fields.keys(), inst=inst, data=data, files=files,
                              initial=self._get_form_initial(inst), partial_update=partial_update)

        errors = form.is_invalid()
        if errors:
            raise DataInvalidException(errors)

        inst = form.save(commit=False)

        can_save_obj = self._can_save_obj(change, inst, form, via)
        if can_save_obj:
            self._pre_save_obj(inst, form, change)
            self._save_obj(inst, form, change)

        if inst.pk:
            for preprocessor in data_postprocessors.get_processors(type(self)):
                data, files = preprocessor(self, form, inst, via, partial_related_update).process_data(data, files)

        if can_save_obj:
            if hasattr(form, 'post_save'):
                form.post_save()

            # Because reverse related validations is performed after save errors check must be evaluated two times
            errors = form.is_invalid()
            if errors:
                raise DataInvalidException(errors)

            self._post_save_obj(inst, form, change)
        return inst

    def _pre_save_obj(self, obj, form, change):
        pass

    def _save_obj(self, obj, form, change):
        raise NotImplementedError

    def _post_save_obj(self, obj, form, change):
        pass


class BaseModelResource(DefaultRESTModelResource, BaseObjectResource):

    register = True
    abstract = True
    form_class = RESTModelForm
    serializer = ModelResourceSerializer
    form_fields = None

    filters = {}
    filter_manager = MultipleFilterManager()
    order_manager = DefaultModelOrderManager()

    def _filter_queryset(self, qs):
        """
        :return: filtered queryset via filter manager if filter manager is not None.
        """
        if self.filter_manager:
            return self.filter_manager.filter(self, qs, self.request)
        else:
            return qs

    def _order_queryset(self, qs):
        """
        :return: ordered queryset via order manager if order manager is not None.
        """
        if self.order_manager:
            return self.order_manager.sort(self, qs, self.request)
        else:
            return qs

    def _get_queryset(self):
        return self.model.objects.all()

    def _get_obj_or_none(self, pk=None):
        if pk or self._get_pk():
            return get_object_or_none(self._get_queryset(), pk=(pk or self._get_pk()))
        else:
            return None

    def _exists_obj(self, **kwargs):
        return self.model.objects.filter(**kwargs).exists()

    def _delete_obj(self, obj):
        obj.delete()

    def _save_obj(self, obj, form, change):
        obj.save()

    def _get_exclude(self, obj=None):
        return []

    def _get_form_class(self, inst):
        return self.form_class

    def _get_name(self):
        return force_text(remove_accent(force_text(self.model._meta.verbose_name_plural)))

    def formfield_for_dbfield(self, db_field, **kwargs):
        if isinstance(db_field, DateTimeField):
            kwargs.update({'form_class': ISODateTimeField})
        return db_field.formfield(**kwargs)

    def _get_instance(self, data):
        # If data contains id this method is update otherwise create
        inst = None
        pk = data.get(self.pk_field_name)
        if pk:
            try:
                try:
                    inst = self._get_queryset().get(pk=pk)
                except (ObjectDoesNotExist, ValueError):
                    if self.model.objects.filter(pk=pk).exists():
                        raise ConflictException
            except ValueError:
                pass
        return inst

    def _get_form_fields(self, obj=None):
        return self.form_fields

    def _obj_name(self, obj):
        return str(obj)

    def _generate_form_class(self, inst, exclude=None):
        exclude = [] if exclude is None else exclude
        exclude = list(self._get_exclude(inst)) + exclude
        form_class = self._get_form_class(inst)
        fields = self._get_form_fields(inst)
        if hasattr(form_class, '_meta') and form_class._meta.exclude:
            exclude.extend(form_class._meta.exclude)
        return rest_modelform_factory(
            self.model, form=form_class, resource_typemapper=self.resource_typemapper,
            auto_related_direct_fields=settings.AUTO_RELATED_DIRECT_FIELDS,
            auto_related_reverse_fields=settings.AUTO_RELATED_REVERSE_FIELDS,
            exclude=exclude, fields=fields,
            formfield_callback=self.formfield_for_dbfield
        )



import os
import requests

import cgi
import mimetypes

import magic  # pylint: disable=E0401

from io import BytesIO

from django.core.exceptions import SuspiciousOperation

from pyston.conf import settings


def request(method, url, **kwargs):
    try:
        from security.transport.security_requests import request

        kwargs['slug'] = 'Pyston - file download'
    except ImportError:
        from requests import request

    return request(method, url, **kwargs)


class RequestDataTooBig(SuspiciousOperation):
    pass


class InvalidResponseStatusCode(SuspiciousOperation):
    pass


def get_content_type_from_filename(filename):
    return mimetypes.guess_type(filename)[0]


def get_content_type_from_file_content(content):
    with magic.Magic(flags=magic.MAGIC_MIME_TYPE) as m:
        mime_type = m.id_buffer(content.read(1024))
        content.seek(0)
        return mime_type


def get_filename_from_content_type(content_type):
    extensions = mimetypes.guess_all_extensions(content_type)
    known_extensions = [ext for exts, name in settings.DEFAULT_FILENAMES for ext in exts]
    extensions.sort(
        key=lambda x: known_extensions.index(x[1:]) if x[1:] in known_extensions else len(known_extensions)
    )
    extension = extensions[0] if extensions else None

    if extension:
        default_filenames = {ext: name for exts, name in settings.DEFAULT_FILENAMES for ext in exts}
        filename = default_filenames.get(extension[1:], settings.DEFAULT_FILENAME)
        return '{}{}'.format(filename, extension)
    else:
        return None


def get_file_name_type_and_content_from_url(url, limit, timeout=1):
    resp = request('get', url, timeout=timeout, stream=True)

    content = BytesIO()
    length = 0

    if resp.status_code != 200:
        raise InvalidResponseStatusCode('Invalid response status code "{}"'.format(resp.status_code))

    for chunk in resp.iter_content(2048):
        content.write(chunk)
        length += len(chunk)
        if length > limit:
            resp.close()
            raise RequestDataTooBig('Requested file is too big')

    content.seek(0)

    params = cgi.parse_header(resp.headers.get('Content-Disposition', ''))[-1]
    filename = os.path.basename(params['filename']) if 'filename' in params else None
    content_type = resp.headers.get('Content-Type', None)

    return filename, content_type, content



# Apply patch only if django is installed
try:
    from django.core.exceptions import ImproperlyConfigured
    try:
        from django.db import models  # NOQA
        from pyston.patch import *  # NOQA
        from pyston.filters.default_filters import * # NOQA
    except ImproperlyConfigured:
        pass
except ImportError as ex:
    pass



from dateutil import parser
import copy

from collections import OrderedDict

from django import forms
from django.core.exceptions import NON_FIELD_ERRORS, ValidationError, ImproperlyConfigured
from django.http.response import Http404
from django.forms.models import ModelFormMetaclass, modelform_factory
from django.utils.translation import ugettext
from django.utils.encoding import force_text, force_str

from chamber.shortcuts import get_object_or_none
from chamber.utils.decorators import classproperty

from pyston.conf import settings as pyston_settings
from pyston.exception import DataInvalidException, RESTException
from pyston.utils.compatibility import (
    get_reverse_field_name, get_model_from_relation, is_reverse_many_to_many, is_reverse_one_to_one,
    is_reverse_many_to_one, delete_cached_value
)
from pyston.utils.helpers import str_to_class


DEFAULT_CODE = 'invalid'


class RESTError(Exception):
    pass


class RESTListError(RESTError):
    """
    List exception is standard list object that can be raised like exception.
    """

    def __init__(self, data=None):
        super(RESTListError, self).__init__()
        self._list = list(data) if data is not None else []

    def __repr__(self):
        return "<{0} {1}>".format(self.__class__.__name__, self._list)

    def __len__(self):
        return len(self._list)

    def __getitem__(self, i):
        return self._list[i]

    def __delitem__(self, i):
        del self._list[i]

    def __setitem__(self, i, val):
        self._list[i] = val

    def __str__(self):
        return str(self._list)

    def insert(self, i, val):
        self._list.insert(i, val)

    def append(self, val):
        self._list.append(val)


class RESTDictError(RESTError):
    """
    Dict exception is standard dict object that can be raised like exception.
    """

    def __init__(self, data=None):
        super(RESTDictError, self).__init__()
        self._dict = dict(data) if data is not None else {}

    def __setitem__(self, key, item):
        self._dict[key] = item

    def __getitem__(self, key):
        return self._dict[key]

    def __contains__(self, key):
        return key in self._dict

    def __repr__(self):
        return '<{0} {1}>'.format(self.__class__.__name__, self._dict)

    def __str__(self):
        return str(self._dict)

    def __len__(self):
        return len(self._dict)

    def __delitem__(self, key):
        del self._dict[key]

    def clear(self):
        return self._dict.clear()

    def has_key(self, k):
        return k in self._dict

    def update(self, *args, **kwargs):
        return self._dict.update(*args, **kwargs)

    def keys(self):
        return self._dict.keys()

    def values(self):
        return self._dict.values()

    def items(self):
        return self._dict.items()

    def pop(self, key):
        return self._dict.pop(key)


class RESTDictIndexError(RESTError):

    def __init__(self, index, data):
        super(RESTDictIndexError, self).__init__()
        self.index = index
        self.data = RESTDictError(data)

    def __repr__(self):
        return "<{0} {1}: {2}>".format(self.__class__.__name__, self.index, self.data)

    def __str__(self):
        return str(self.data)


class RESTValidationError(RESTError):

    def __init__(self, message, code=None):
        if isinstance(message, RESTValidationError):
            self.message = message.message
            self.code = message.code
        else:
            self.message = message
            self.code = code or DEFAULT_CODE

    def __str__(self):
        return self.message


class AllFieldsUniqueValidationModelForm(forms.ModelForm):

    def validate_unique(self):
        try:
            self.instance.validate_unique()
        except ValidationError as e:
            self._update_errors(e)


class RelatedField:

    is_reverse = False

    def __init__(self, resource_class=None):
        self.resource_class = resource_class

    def _get_resource(self, model, request):
        from pyston.serializer import get_resource_class_or_none

        resource_class = self.resource_class

        if isinstance(resource_class, str):
            resource_class = str_to_class(resource_class)
        elif resource_class is None:
            resource_class = get_resource_class_or_none(model)
        assert resource_class, 'Missing resource for model {}'.format(model)
        return resource_class(request)

    def _flat_object_to_pk(self, pk_field_name, data):
        if isinstance(data, dict):
            try:
                return data[pk_field_name]
            except KeyError:
                raise RESTValidationError(
                    ugettext('Data must contain primary key: {}').format(pk_field_name), code='invalid_structure'
                )
        else:
            return data

    def _delete_related_object(self, resource, data, via):
        try:
             resource.delete_obj_with_pk(self._flat_object_to_pk(resource.pk_field_name, data), via)
        except DataInvalidException as ex:
            raise ex.errors
        except RESTException as ex:
            raise RESTValidationError(ex.message)
        except Http404:
            raise RESTValidationError(ugettext('Object does not exist'), code='invalid_structure')

    def _create_or_update_related_object(self, resource, data, via, partial_update):
        if not isinstance(data, dict):
            raise RESTValidationError(ugettext('Data must be object'), code='invalid_structure')

        try:
            return resource.create_or_update(resource.update_deserialized_data(data), via, partial_update)
        except DataInvalidException as ex:
            raise ex.errors
        except RESTException as ex:
            raise RESTValidationError(ex.errors)

    def create_update_or_remove(self, parent_inst, data, via, request, partial_update, form):
        raise NotImplementedError

    def _add_parent_inst_to_obj_data(self, parent_inst, field_name, data):
        data = data.copy()
        data[field_name] = parent_inst.pk
        return data

    def _create_and_return_new_object_pk_list(self, resource, parent_inst, via, data, partial_update,
                                              created_via_field_name=None):
        errors = RESTListError()
        result = []
        for i, obj_data in enumerate(data):
            if not isinstance(obj_data, dict):
                obj_data = {resource.pk_field_name: force_text(obj_data)}

            try:
                if created_via_field_name:
                    obj_data = self._add_parent_inst_to_obj_data(parent_inst, created_via_field_name, obj_data)

                if set(obj_data.keys()) ^ {resource.pk_field_name}:
                    result.append(self._create_or_update_related_object(resource, obj_data, via, partial_update).pk)
                else:
                    result.append(obj_data[resource.pk_field_name])
            except RESTDictError as ex:
                errors.append(RESTDictIndexError(i, ex))
            except RESTError as ex:
                errors.append(RESTDictIndexError(i, {'error': ex}))

        if errors:
            raise errors
        return result


class DirectRelatedField(RelatedField):

    def __init__(self, field_name, form_field=None, resource_class=None):
        super(DirectRelatedField, self).__init__(resource_class)
        self.form_field = form_field
        self.field_name = field_name


class SingleRelatedField(DirectRelatedField):

    def create_update_or_remove(self, parent_inst, data, via, request, partial_update, form):
        if isinstance(data, dict):
            resource = self._get_resource(self.form_field.queryset.model, request)
            return self._create_or_update_related_object(resource, data, via, partial_update).pk
        else:
            return data


class MultipleRelatedField(DirectRelatedField):

    def _update_related_objects(self, resource, parent_inst, via, data, partial_update, form):
        if isinstance(data, (tuple, list)):
            return self._create_and_return_new_object_pk_list(resource, parent_inst, via, data, partial_update)
        else:
            raise RESTValidationError(ugettext('Data must be a collection'), code='invalid_structure')

    def create_update_or_remove(self, parent_inst, data, via, request, partial_update, form):
        resource = self._get_resource(self.form_field.queryset.model, request)
        return self._update_related_objects(resource, parent_inst, via, data, partial_update, form)


class MultipleStructuredRelatedField(MultipleRelatedField):

    def _remove_related_objects(self, resource, parent_inst, via, data, values):
        errors = RESTListError()
        result = [force_text(val) for val in values]
        for i, obj in enumerate(data):
            try:
                pk = force_text(self._flat_object_to_pk(resource.pk_field_name, obj))
                if pk in result:
                    result.remove(pk)
                else:
                    errors.append({'error': ugettext('Object does not exist in selected data'), '_index': i})
            except RESTDictError as ex:
                errors.append(RESTDictIndexError(i, ex))
            except RESTError as ex:
                errors.append(RESTDictIndexError(i, {'error': ex}))

        if errors:
            raise errors
        return result

    def _add_related_objects(self, resource, parent_inst, via, data, values, partial_update):
        if isinstance(data, (tuple, list)):
            return values + self._create_and_return_new_object_pk_list(resource, parent_inst, via, data, partial_update)
        else:
            raise RESTValidationError(ugettext('Data must be a collection'), code='invalid_structure')

    def _add_and_remove_structured_objects(self, resource, parent_inst, via, data, partial_update, form):
        errors = RESTDictError()
        values = self.form_field.prepare_value(form.initial.get(self.field_name, self.form_field.initial)) or []
        if 'remove' in data:
            try:
                values = self._remove_related_objects(resource, parent_inst, via, data.get('remove'), values)
            except RESTError as ex:
                errors['remove'] = ex
        if 'add' in data:
            try:
                values = self._add_related_objects(resource, parent_inst, via, data.get('add'), values, partial_update)
            except RESTError as ex:
                errors['add'] = ex
        if errors:
            raise errors
        else:
            return values

    def _update_structured_object(self, resource, parent_inst, via, data, partial_update, form):
        if 'set' in data:
            if {'remove', 'add'} & set(data.keys()):
                raise RESTValidationError(
                    ugettext('set cannot be together with add or remove'), code='invalid_structure'
                )
            try:
                return super(MultipleStructuredRelatedField, self)._update_related_objects(
                    resource, parent_inst, via, data.get('set'), partial_update, form
                )
            except RESTError as ex:
                raise RESTDictError({'set': ex})
        else:
            return self._add_and_remove_structured_objects(resource, parent_inst, via, data, partial_update, form)

    def _update_related_objects(self, resource, parent_inst, via, data, partial_update, form):
        if isinstance(data, dict):
            return self._update_structured_object(resource, parent_inst, via, data, partial_update, form)
        else:
            return super(MultipleStructuredRelatedField, self)._update_related_objects(
                resource, parent_inst, via, data, partial_update, form
            )


class ReverseField(RelatedField):

    is_reverse = True

    def __init__(self, reverse_field_name, extra_data=None, resource_class=None):
        super(ReverseField, self).__init__(resource_class)
        self.reverse_field_name = reverse_field_name
        self.extra_data = extra_data

    def _get_extra_data(self, parent_inst):
        return self.extra_data


class ReverseSingleField(ReverseField):

    def _get_obj_or_none(self, model, parent_inst, field_name):
        return None

    def _remove(self, resource, parent_inst, related_obj, field_name, via):
        self._delete_related_object(
            resource, {resource.pk_field_name: related_obj.pk}, via
        )

    def _create_or_update(self, resource, parent_inst, related_obj, field_name, via, data, partial_update):
        if not isinstance(data, dict):
            obj_data = {resource.pk_field_name: force_text(data)}
        else:
            obj_data = data.copy()

        extra_data = self._get_extra_data(parent_inst)
        if extra_data:
            obj_data.update(extra_data)

        if resource.pk_field_name not in obj_data and related_obj:
            obj_data[resource.pk_field_name] = related_obj.pk
        obj_data[field_name] = parent_inst.pk
        return self._create_or_update_related_object(resource, obj_data, via, partial_update)

    def create_update_or_remove(self, parent_inst, data, via, request, partial_update, form):
        model = get_model_from_relation(parent_inst.__class__, self.reverse_field_name)
        field_name = get_reverse_field_name(parent_inst.__class__, self.reverse_field_name)
        resource = self._get_resource(model, request)
        related_obj = self._get_obj_or_none(model, parent_inst, field_name)
        if data is None and related_obj:
            self._remove(resource, parent_inst, related_obj, field_name, via)
            return None
        elif data is not None:
            return self._create_or_update(resource, parent_inst, related_obj, field_name, via, data, partial_update)


class ReverseOneToOneField(ReverseSingleField):

    def _get_obj_or_none(self, model, parent_inst, field_name):
        return get_object_or_none(model, **{field_name: parent_inst.pk})

    def _remove(self, resource, parent_inst, related_obj, field_name, via):
        super(ReverseOneToOneField, self)._remove(resource, parent_inst, related_obj, field_name, via)
        delete_cached_value(parent_inst, self.reverse_field_name)

    def _create_or_update(self, resource, parent_inst, related_obj, field_name, via, data, partial_update):
        obj = super(ReverseOneToOneField, self)._create_or_update(resource, parent_inst, related_obj, field_name, via,
                                                                  data, partial_update)
        setattr(parent_inst, self.reverse_field_name, obj)
        return obj


class ReverseManyField(ReverseField):

    is_deleted_not_selected_objects = True

    def _add_parent_inst_to_obj_data(self, parent_inst, field_name, data):
        if is_reverse_many_to_many(parent_inst, self.reverse_field_name):
            data = data.copy()
            data[field_name] = {'add': [parent_inst.pk]}
            return data
        else:
            return super(ReverseManyField, self)._add_parent_inst_to_obj_data(parent_inst, field_name, data)

    def _delete_reverse_objects(self, resource, data, via):
        errors = RESTListError()
        for i, obj_data in enumerate(data):
            try:
                self._delete_related_object(
                    resource,
                    resource.update_deserialized_data(obj_data) if isinstance(obj_data, dict) else obj_data,
                    via
                )
            except RESTDictError as ex:
                errors.append(RESTDictIndexError(i, ex))
            except RESTError as ex:
                errors.append(RESTDictIndexError(i, {'error': ex}))

        if errors:
            raise errors

    def create_update_or_remove(self, parent_inst, data, via, request, partial_update, form):
        model = get_model_from_relation(parent_inst.__class__, self.reverse_field_name)
        field_name = get_reverse_field_name(parent_inst.__class__, self.reverse_field_name)
        resource = self._get_resource(model, request)
        return self._update_reverse_related_objects(resource, model, parent_inst, field_name, via, data, partial_update)

    def _update_reverse_related_objects(self, resource, model, parent_inst, field_name, via, data, partial_update):
        if isinstance(data, (tuple, list)):
            new_object_pks = self._create_and_return_new_object_pk_list(resource, parent_inst, via, data,
                                                                        partial_update, field_name)
            # This is not optimal solution but is the most universal
            if self.is_deleted_not_selected_objects:
                self._delete_reverse_objects(
                    resource, resource._get_queryset().filter(**{field_name: parent_inst}).exclude(
                        pk__in=new_object_pks
                    ).values_list('pk', flat=True),
                    via
                )
            return model.objects.filter(pk__in=new_object_pks)
        else:
            raise RESTValidationError(ugettext('Data must be a collection'), code='invalid_structure')


class ReverseStructuredManyField(ReverseManyField):

    def _remove_reverse_related_objects(self, resource, parent_inst, via, data, field_name):
        if isinstance(data, (tuple, list)):
            self._delete_reverse_objects(resource, data, via)
        else:
            raise RESTValidationError(ugettext('Data must be a collection'), code='invalid_structure')

    def _add_reverse_related_objects(self, resource, model, parent_inst, via, data, partial_update, field_name):
        if isinstance(data, (tuple, list)):
            new_object_pks = self._create_and_return_new_object_pk_list(resource, parent_inst, via, data,
                                                                        partial_update, field_name)
            return model.objects.filter(pk__in=new_object_pks)
        else:
            raise RESTValidationError(ugettext('Data must be a collection'), code='invalid_structure')

    def _add_and_remove_structured_objects(self, resource, model, parent_inst, field_name, via, data, partial_update):
        errors = RESTDictError()
        if 'remove' in data:
            try:
                objects_qs = model.objects.none()
                self._remove_reverse_related_objects(resource, parent_inst, via, data.get('remove'), field_name)
            except RESTError as ex:
                errors['remove'] = ex
        if 'add' in data:
            try:
                objects_qs = self._add_reverse_related_objects(
                    resource, model, parent_inst, via, data.get('add'), partial_update, field_name
                )
            except RESTError as ex:
                errors['add'] = ex
        if errors:
            raise errors
        else:
            return objects_qs

    def _update_structured_object(self, resource, model, parent_inst, field_name, via, data, partial_update):
        if 'set' in data:
            if {'remove', 'add'} & set(data.keys()):
                raise RESTValidationError(
                    ugettext('set cannot be together with add or remove'), code='invalid_structure'
                )
            try:
                return super(ReverseStructuredManyField, self)._update_reverse_related_objects(
                    resource, model, parent_inst, field_name, via, data.get('set'), partial_update
                )
            except RESTError as ex:
                raise RESTDictError({'set': ex})
        else:
            return self._add_and_remove_structured_objects(resource, model, parent_inst, field_name, via, data,
                                                           partial_update)

    def _update_reverse_related_objects(self, resource, model, parent_inst, field_name, via, data, partial_update):
        if isinstance(data, dict) and data and set(data.keys()) <= {'add', 'remove', 'set'}:
            return self._update_structured_object(resource, model, parent_inst, field_name, via, data, partial_update)
        else:
            return super(ReverseStructuredManyField, self)._update_reverse_related_objects(
                resource, model, parent_inst, field_name, via, data, partial_update
            )


class ISODateTimeField(forms.DateTimeField):

    def strptime(self, value, format):
        return parser.parse(force_str(value))


class RESTFormMixin:

    def __init__(self, *args, **kwargs):
        self.origin_initial = kwargs.get('initial', {})
        self.partial_update = kwargs.pop('partial_update', False)
        super(RESTFormMixin, self).__init__(*args, **kwargs)

    def _parse_rest_errors(self, errors):
        if isinstance(errors, RESTError):
            return errors
        elif isinstance(errors, dict):
            return RESTDictError({k: self._parse_rest_errors(v) for k, v in errors.items()})
        else:
            return RESTValidationError(list(errors.as_data()[0])[0], errors.as_data()[0].code)

    def is_invalid(self):
        """
        Validate input data. It uses django forms
        """
        errors = RESTDictError() if self.is_valid() else self._parse_rest_errors(self.errors)

        if '__all__' in errors:
            del errors['__all__']

        non_field_errors = self.non_field_errors()
        if non_field_errors:
            errors['non-field-errors'] = self._parse_rest_errors(non_field_errors)

        if errors:
            return errors

        return False

    def is_valid(self):
        self.origin_data = self.data
        # For partial update is used model values
        # for other cases is used only default values and redefined initial values during form initialization
        self._merge_from_initial(self.initial if self.partial_update else self.origin_initial)
        return super(RESTFormMixin, self).is_valid()

    """
    Subclass of `forms.ModelForm` which makes sure
    that the initial values are present in the form
    data, so you don't have to send all old values
    for the form to actually validate. Django does not
    do this on its own, which is really annoying.
    """
    def _merge_from_initial(self, initial):
        self.data = self.data.copy()
        filt = lambda v: v not in self.data.keys()
        for field_name in filter(filt, self.fields.keys()):
            field = self.fields[field_name]
            self.data[field_name] = field.prepare_value(initial.get(field_name, field.initial))

    def add_error(self, field, error):
        """
        Standard django form does not support more structured errors
        (for example error dict that contains another error dict).
        For this purpose pyston creates RESTError class and we must rewrite this method to allow these
        complex error messages.
        """
        if isinstance(error, RESTError):
            if not field:
                raise ValueError('Field must be set for RESTError')
            self._errors[field] = error
        else:
            if not isinstance(error, ValidationError):
                # Normalize to ValidationError and let its constructor
                # do the hard work of making sense of the input.
                error = ValidationError(error)

            if hasattr(error, 'error_dict'):
                if field is not None:
                    raise TypeError(
                        "The argument `field` must be `None` when the `error` "
                        "argument contains errors for multiple fields."
                    )
                else:
                    error = error.error_dict
            else:
                error = {field or NON_FIELD_ERRORS: error.error_list}

            for field, error_list in error.items():
                if field not in self.errors:
                    if field == NON_FIELD_ERRORS:
                        self._errors[field] = self.error_class(error_class='nonfield')
                    else:
                        self._errors[field] = self.error_class()
                self._errors[field].extend(error_list)
                if field in self.cleaned_data:
                    del self.cleaned_data[field]


def get_resource_class( model, resource_typemapper):
    from pyston.serializer import get_resource_class_or_none

    return get_resource_class_or_none(model, resource_typemapper)


def direct_related_fields_for_model(base_fields, resource_typemapper=None):
    """
    Generate direct from form fields.
    """

    related_fields = OrderedDict()
    for name, field in base_fields.items():
        if isinstance(field, forms.ModelMultipleChoiceField):
            resource_class = get_resource_class(field.queryset.model, resource_typemapper)
            if resource_class:
                related_fields[name] = MultipleStructuredRelatedField(
                    name, form_field=field, resource_class=resource_class
                )
        elif isinstance(field, forms.ModelChoiceField):
            resource_class = get_resource_class(field.queryset.model, resource_typemapper)
            if resource_class:
                related_fields[name] = SingleRelatedField(
                    name, form_field=field, resource_class=resource_class
                )
    return related_fields


def reverse_related_fields_for_model(model, fields=None, exclude=None, resource_typemapper=None):
    """
    Generate reverse related fields for concrete model.
    """

    field_list = []
    opts = model._meta
    reverse_fields = [
        f for f in opts.get_fields()
        if (f.one_to_many or f.one_to_one or f.many_to_many)
        and f.auto_created and not f.concrete
    ]
    for f in reverse_fields:
        if fields is not None and f.name not in fields:
            continue
        if exclude and f.name in exclude:
            continue
        if not f.related_name:
            continue

        resource_class = get_resource_class(get_model_from_relation(model, f.name), resource_typemapper)

        if resource_class and (is_reverse_many_to_many(model, f.name) or is_reverse_many_to_one(model, f.name)):
            field_list.append((f.name, ReverseStructuredManyField(f.name, resource_class=resource_class)))
        elif resource_class and is_reverse_one_to_one(model, f.name):
            field_list.append((f.name, ReverseOneToOneField(f.name, resource_class=resource_class)))

    return OrderedDict(field_list)


class RESTModelFormOptions:

    def __init__(self, options=None):
        self.resource_typemapper = getattr(options, 'resource_typemapper', None)
        self.auto_related_direct_fields = getattr(options, 'auto_related_direct_fields', False)
        self.auto_related_reverse_fields = getattr(options, 'auto_related_reverse_fields', False)


class RESTFormMetaclass(ModelFormMetaclass):
    """
    Form metaclass that improves django model form with reverse fields.
    """

    def __new__(cls, name, bases, attrs):
        # Collect related fields from current class.
        current_related_fields = []
        for key, value in list(attrs.items()):
            if isinstance(value, RelatedField):
                current_related_fields.append((key, value))
                attrs.pop(key)
                if not value.is_reverse and value.form_field:
                    attrs[key] = value.form_field

        attrs['declared_related_fields'] = OrderedDict(current_related_fields)

        new_class = super(RESTFormMetaclass, cls).__new__(cls, name, bases, attrs)

        rest_opts = new_class._rest_meta = RESTModelFormOptions(getattr(new_class, 'RESTMeta', None))

        # Walk through the MRO.
        declared_related_fields = OrderedDict()
        for base in reversed(new_class.__mro__):
            # Collect fields from base class.
            if hasattr(base, 'declared_related_fields'):
                declared_related_fields.update(base.declared_related_fields)

            # Field shadowing.
            for attr, value in base.__dict__.items():
                if value is None and attr in declared_related_fields:
                    declared_related_fields.pop(attr)

        opts = new_class._meta

        new_class.base_related_fields = declared_related_fields
        new_class.declared_related_fields = declared_related_fields

        if opts.model:
            reverse_related_fields = reverse_related_fields_for_model(
                opts.model, opts.fields, opts.exclude, rest_opts.resource_typemapper
            ) if rest_opts.auto_related_reverse_fields else {}

            direct_related_fields = direct_related_fields_for_model(
                new_class.base_fields, rest_opts.resource_typemapper
            ) if rest_opts.auto_related_direct_fields else {}
            related_fields = reverse_related_fields
            related_fields.update(direct_related_fields)
            related_fields.update(declared_related_fields)

            for name, field in related_fields.items():
                if not field.is_reverse and name not in new_class.base_fields:
                    raise ValueError("Missing django form field for related field field '{}'".format(name))
                elif not field.is_reverse:
                    field.form_field = new_class.base_fields[name]

        else:
            related_fields = declared_related_fields

        new_class.base_related_fields = related_fields
        new_class.declared_related_fields = related_fields

        return new_class


class RESTModelForm(RESTFormMixin, AllFieldsUniqueValidationModelForm, metaclass=RESTFormMetaclass):

    def __init__(self, *args, **kwargs):
        super(RESTModelForm, self).__init__(*args, **kwargs)
        self.related_fields = copy.deepcopy(self.base_related_fields)

    def _reverse_fields_clean(self):
        """
        Call clean methods on reverse fields.
        """
        for name, field in self.related_fields.items():
            if field.is_reverse and hasattr(self, 'clean_{}'.format(name)):
                try:
                    getattr(self, 'clean_{}'.format(name))()
                except (ValidationError, RESTError) as e:
                    self.add_error(name, e)

    def _post_save_clean(self):
        """
        Reverse related fields can be validated after current object save.
        """
        self._reverse_fields_clean()

    def _pre_save(self, obj):
        pass

    def _post_save(self, obj):
        pass

    def _set_post_save(self, commit, obj):
        """
        Default django form is updated with pre, post save methods and post save clean.
        For this purpose is used django default method save_m2m. But for cleaner code can be used method post_save.
        """
        if commit:
            self._post_save(obj)
        else:
            self.old_save_m2m = self.save_m2m

            def post_save():
                self.old_save_m2m()
                self._post_save_clean()
                self._post_save(obj)
            self.save_m2m = post_save
            self.post_save = post_save

    def save(self, commit=True):
        """
        Updates default save method with pre and post save methods.
        """
        obj = super(RESTModelForm, self).save(commit=False)
        self._pre_save(obj)
        if commit:
            obj.save()
        self._set_post_save(commit, obj)
        return obj


def rest_modelform_factory(model, form=RESTModelForm, form_factory=modelform_factory, resource_typemapper=None,
                           auto_related_direct_fields=None, auto_related_reverse_fields=None, **kwargs):
    """
    Purpose of the rest factory form method is prepare RESTMeta for standard django model form.
    The final form is created using a factory defined in form_factory argument.
    :param model: Django model class
    :param form: base model form class
    :param form_factory: provides posibility to change default modelform_factory
    :param resource_typemapper: resource typemapper defines which resources will be used for generation reverse fields
    :param auto_related_direct_fields: defines if form will auto generate direct related REST fields
    :param auto_related_reverse_fields: defines if form will auto generate reverse related REST fields
    :param kwargs: kwargs of form_factory
    :return: django model form class
    """
    attrs = {}
    if resource_typemapper:
        attrs['resource_typemapper'] = resource_typemapper
    if auto_related_direct_fields:
        attrs['auto_related_direct_fields'] = auto_related_direct_fields
    if auto_related_reverse_fields:
        attrs['auto_related_reverse_fields'] = auto_related_reverse_fields

    parent = (object,)
    if hasattr(form, 'RESTMeta'):
        parent = (form.RESTMeta, object)

    RESTMeta = type(str('RESTMeta'), parent, attrs)

    # Give this new form class a reasonable name.
    form_class_name = model.__name__ + str('Form')

    # Class attributes for the new form class.
    form_class_attrs = {
        'RESTMeta': RESTMeta,
    }
    # Instantiate type(form) in order to use the same metaclass as form.
    return form_factory(model, form=type(form)(form_class_name, (form,), form_class_attrs), **kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-10-02 07:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0003_auto_20170729_2305'),
    ]

    operations = [
        migrations.AddField(
            model_name='issue',
            name='tags',
            field=models.TextField(blank=True, null=True, verbose_name='tags'),
        ),
    ]



from django import forms
from django.db.models import F, Q

from pyston.converters import XMLConverter
from pyston.resource import BaseModelResource, BaseResource, BaseObjectResource
from pyston.response import RESTCreatedResponse, RESTOkResponse
from pyston.serializer import SerializableObj
from pyston.forms import (
    RESTModelForm, ReverseOneToOneField, ReverseManyField, RESTValidationError, SingleRelatedField,
    MultipleRelatedField
)
from pyston.forms.postgres import RESTSimpleArrayField
from pyston.filters.default_filters import SimpleEqualFilter, BooleanFilterMixin

from .models import Issue, User
from .serializable import CountIssuesPerUserTable, CountWatchersPerIssue


class OvertimeIssuesFilter(BooleanFilterMixin, SimpleEqualFilter):

    def get_filter_term(self, value, operator, request):
        filter_term = Q(**{
            'solving_issue__in': Issue.objects.filter(logged_minutes__gt=F('estimate_minutes')).values('pk')
        })
        return filter_term if value else ~filter_term


class IssueResource(BaseModelResource):

    model = Issue
    fields = ('id', 'created_at', '_obj_name', 'name', ('created_by', ('id', 'contract', 'created_at')), 'solver',
              'leader', 'watched_by', 'logged_minutes')
    detailed_fields = ('id', 'created_at', '_obj_name', 'name', ('created_by', ('id', 'contract',)), 'solver',
                       'leader', 'watched_by')
    general_fields = ('id', '_obj_name', 'name', ('created_by', ('id', 'contract', 'created_at')), 'watched_by',
                      'short_description')

    converter_classes = (
        'pyston.converters.JSONConverter',
        XMLConverter,
    )
    can_create_obj = True
    can_read_obj = True
    can_update_obj = True
    can_delete_obj = True


class UserResource(BaseModelResource):

    model = User
    DATA_KEY_MAPPING = {
        'created_at': 'createdAt',
        'solving_issue': 'solvingIssue',
        'first_name': 'firstName',
        'last_name': 'lastName',
        'is_superuser': 'isSuperuser',
        'watched_issues': 'watchedIssues',
        'created_issues': 'createdIssues',
        'manual_created_date': 'manualCreatedDate',
        'watched_issues_count': 'watchedIssuesCount',
    }
    can_create_obj = True
    can_read_obj = True
    can_update_obj = True
    can_delete_obj = True
    extra_order_fields = ()
    extra_filter_fields = ()
    filters = {
        'issues__overtime': OvertimeIssuesFilter
    }


class ExtraResource(BaseResource):

    def get(self):
        return {'extra': 1}


class CountIssuesPerUserResource(BaseResource):

    def get(self):
        return CountIssuesPerUserTable()


class CountWatchersPerIssueResource(BaseResource):

    def get(self):
        return [CountWatchersPerIssue(issue) for issue in Issue.objects.all()]


class TestTextObject(SerializableObj):

    def __init__(self, fiz_baz):
        self.fiz_baz = fiz_baz

    class RESTMeta:
        fields = ('fiz_baz',)


class TestTextObjectCamelCaseResource(BaseObjectResource):

    model = TestTextObject
    register = True

    can_read_obj = True

    DATA_KEY_MAPPING = {
        'fiz_baz': 'fizBaz',
    }


class TestCamelCaseResource(BaseResource):

    DATA_KEY_MAPPING = {
        'bar_baz': 'barBaz',
        'foo_bar': 'fooBar',
    }

    def get(self):
        connected = TestTextObject('test object property content')
        return {
            'foo_bar': 'foo bar',
            'connected': connected,
        }

    def post(self):
        data = self.get_dict_data()
        return RESTCreatedResponse({'bar_baz': data.get('bar_baz')})


class UserForm(RESTModelForm):

    watched_issues = ReverseManyField('watched_issues')
    created_issues_renamed = ReverseManyField('created_issues')
    solving_issue_renamed = ReverseOneToOneField('solving_issue')
    leading_issue_renamed = ReverseOneToOneField('leading_issue')

    def clean_created_issues_renamed(self):
        created_issues = self.cleaned_data.get('created_issues_renamed')
        if created_issues and any(issue.name == 'invalid' for issue in created_issues):
            raise RESTValidationError('Invalid issue name')


class UserWithFormResource(BaseModelResource):

    register = False
    model = User
    form_class = UserForm
    can_create_obj = True
    can_read_obj = True
    can_update_obj = True
    can_delete_obj = True

    fields = ('user_email',)

    def user_email(self, obj):
        return obj.email


class IssueForm(RESTModelForm):

    created_by = SingleRelatedField('created_by')
    leader = SingleRelatedField('leader')
    another_users = MultipleRelatedField('watched_by', form_field=forms.ModelMultipleChoiceField(
        queryset=User.objects.all(), required=False
    ))
    tags_list = RESTSimpleArrayField(label='tags', base_field=forms.CharField(max_length=5), required=False)

    def save(self, commit=True):
        instance = super().save(commit=False)
        tags_list = self.cleaned_data.get('tags_list')
        if tags_list:
            instance.tags = '|'.join(tags_list)
        if commit:
            instance.save()
        return instance

    class Meta:
        exclude = ('tags',)


class IssueWithFormResource(BaseModelResource):

    register = False
    model = Issue
    form_class = IssueForm
    can_create_obj = True
    can_read_obj = True
    can_update_obj = True
    can_delete_obj = True

    fields = ('id', 'creator')
    resource_typemapper = {
        User: UserWithFormResource,
    }

    def creator(self, obj):
        return obj.created_by


import factory
from factory import fuzzy

from app import models


class UserFactory(factory.django.DjangoModelFactory):

    first_name = factory.Sequence(lambda n: 'John{0}'.format(n))
    last_name = factory.Sequence(lambda n: 'Doe{0}'.format(n))
    email = factory.Sequence(lambda n: 'customer_{0}@example.com'.format(n).lower())

    class Meta:
        model = models.User


class IssueFactory(factory.django.DjangoModelFactory):

    name = factory.fuzzy.FuzzyText(length=100)
    created_by = factory.SubFactory('app.tests.factories.UserFactory')
    leader = factory.SubFactory('app.tests.factories.UserFactory')

    class Meta:
        model = models.Issue


import re

import pyparsing as pp

from django.utils.safestring import mark_safe
from django.utils.translation import ugettext

from pyston.utils import LOOKUP_SEP

from .default_filters import OPERATORS
from .exceptions import FilterValueError, OperatorFilterError, FilterIdentifierError
from .utils import LOGICAL_OPERATORS


class FilterParserError(Exception):
    """
    Exception that is raised if filter input is invalid.
    """
    pass


class Condition:
    """
    Logical condition tree node.
    """

    def __init__(self, is_composed):
        self.is_composed = is_composed


class ComposedCondition(Condition):
    """
    Composed logical condition tree node. Contains operator (AND, OR, NOT), left and right condition that can be Term
    or ComposedCondition.
    """

    def __init__(self, operator_slug, condition_right, condition_left=None):
        super(ComposedCondition, self).__init__(True)
        self.operator_slug = operator_slug
        self.condition_left = condition_left
        self.condition_right = condition_right
        self.logical_condition = True


class Term(Condition):
    """
    Simple term. Contains operator (=, >, <, =>, contains, in, all, etc.), filter identifiers, value and source input
    value which is used to assemble error messages.
    """

    def __init__(self, operator_slug, identifiers, value, source):
        super(Term, self).__init__(False)
        self.operator_slug = operator_slug
        self.identifiers = identifiers
        self.value = value
        self.source = source


class FilterParser:
    """
    Abstract filter parser.
    """

    def parse(self, request):
        """
        :param request: Django HTTP request.
        :return: returns conditions tree or None.
        """
        raise NotImplementedError


class DefaultFilterParser(FilterParser):
    """
    Parser for complex filter terms.
    E.q.:
       /api/user?filter=created_at__moth=5 AND NOT (contract=null OR first_name='Petr')
    """

    ALLOWED_OPERATORS = (
        '=',
        '!=',
        '>=',
        '<=',
        '>',
        '<',
        '[a-z]+'
    )
    OPERATORS_MAPPING = {
        '=': OPERATORS.EQ,
        '!=': OPERATORS.NEQ,
        '<': OPERATORS.LT,
        '>': OPERATORS.GT,
        '>=': OPERATORS.GTE,
        '<=': OPERATORS.LTE,
    }
    VALUE_MAPPERS = {
        'null': None
    }

    def _clean_value(self, value):
        if isinstance(value, list):
            return [self._clean_value(v) for v in value]
        else:
            return self.VALUE_MAPPERS.get(value, value)

    def _clean_operator(self, operator_slug):
        return self.OPERATORS_MAPPING.get(operator_slug, operator_slug.lower())

    def _parse_to_conditions(self, parsed_result_list, condition_positions, condition, input):
        def _parse_to_conditions_recursive(term):
            if len(term) == 2 and term[0] == LOGICAL_OPERATORS.NOT:
                return LogicalCondition(LOGICAL_OPERATORS.NOT, _parse_to_conditions(term[1]))
            elif len(term) == 3 and term[1] in {LOGICAL_OPERATORS.AND, LOGICAL_OPERATORS.OR}:
                return ComposedCondition(
                    term[1],
                    _parse_to_conditions_recursive(term[2]),
                    _parse_to_conditions_recursive(term[0])
                )
            else:
                position = condition_positions.pop(0)
                from_position, to_position = next(condition.scanString(input[position:]))[1:]
                return Term(
                    self._clean_operator(term[1]),
                    term[0],
                    self._clean_value(term[2]),
                    input[position:][from_position:to_position]
                )
        return _parse_to_conditions_recursive(parsed_result_list)

    def parse(self, request):
        input = request._rest_context.get('filter')
        if not input:
            return None

        condition_positions = []

        operator = pp.Regex('|'.join(self.ALLOWED_OPERATORS))
        number = pp.Regex(r"[+-]?\d+(:?\.\d*)?(:?[eE][+-]?\d+)?")

        AND = pp.Literal(LOGICAL_OPERATORS.AND)
        OR = pp.Literal(LOGICAL_OPERATORS.OR)
        NOT = pp.Literal(LOGICAL_OPERATORS.NOT)

        identifier = pp.Regex(r"[a-zA-Z]+[a-zA-Z0-9]*(_[a-zA-Z0-9]+)*")
        identifiers = pp.Group(pp.delimitedList(identifier, delim="__", combine=False))

        comparison_term = pp.Forward()
        list_term = (
            pp.Group(
                pp.Suppress('[') + pp.delimitedList(comparison_term, delim=",", combine=False) + pp.Suppress(']')
            ) |
            pp.Group(
                pp.Suppress('(') + pp.delimitedList(comparison_term, delim=",", combine=False) + pp.Suppress(')')
            ) |
            pp.Group(
                pp.Suppress('{') + pp.delimitedList(comparison_term, delim=",", combine=False) + pp.Suppress('}')
            )
        )
        string = (
            pp.QuotedString("'", escChar='\\', unquoteResults=True) | pp.QuotedString('"', escChar='\\',
                                                                                      unquoteResults=True)
        )
        null = pp.Literal('null').setParseAction(lambda s,l,t: None)
        boolean = pp.Regex('|'.join(('true', 'false'))).setParseAction(lambda s, l, t: t[0] == 'true')

        comparison_term << (string | number | list_term | null | boolean)

        condition = pp.Group(identifiers + operator + comparison_term).setResultsName('condition')
        condition.setParseAction(lambda s, loc, tocs: condition_positions.append(loc))

        expr = pp.operatorPrecedence(
            condition, [
                (NOT, 1, pp.opAssoc.RIGHT,),
                (AND, 2, pp.opAssoc.LEFT,),
                (OR, 2, pp.opAssoc.LEFT,),
            ]
        )

        try:
            return self._parse_to_conditions(
                expr.parseString(input, parseAll=True).asList()[0], list(condition_positions), condition, input
            )
        except pp.ParseException as ex:
            raise FilterParserError(
                mark_safe(ugettext('Invalid filter value "{}"').format(input))
            )


class FlatAndFilterParser(FilterParser):
    """
    Helper used to implement parsers that join terms only with AND operator.
    """

    def _parse_to_composed_conditions(self, conditions_list):
        if len(conditions_list) == 1:
            return conditions_list[0]
        else:
            return ComposedCondition(LOGICAL_OPERATORS.AND, self._parse_to_composed_conditions(conditions_list[1:]),
                                     conditions_list[0])


class QueryStringFilterParser(FlatAndFilterParser):
    """
    Simple query string parser that parse input request query string to the conditions joined with AND operator.
    E.q.:
       /api/user?created_at__moth=5&contract=__none__
    """

    VALUE_MAPPERS = {
        '__none__': None
    }
    MULTIPLE_VALUES_OPERATORS = {
        OPERATORS.IN, OPERATORS.ALL
    }

    def _clean_multiple_values(self, operator_slug, value):
        for pattern in ('\[(.*)\]', '\((.*)\)', '\{(.*)\}'):
            m = re.compile(pattern).match(value)
            if m:
                return [self._clean_simple_value(v) for v in m.group(1).split(',')] if m.group(1) else []

        raise FilterParserError('Value must be in list "[]", tuple "()" or set "{}" format split with char ",".')

    def _clean_simple_value(self, value):
        return self.VALUE_MAPPERS.get(value, value)

    def _clean_value(self, operator_slug, value):
        if operator_slug.lower() in self.MULTIPLE_VALUES_OPERATORS:
            return self._clean_multiple_values(operator_slug, value)
        else:
            return self._clean_simple_value(value)

    def _parse_to_composed_conditions(self, conditions_list):
        if len(conditions_list) == 1:
            return conditions_list[0]
        else:
            return ComposedCondition(LOGICAL_OPERATORS.AND, self._parse_to_composed_conditions(conditions_list[1:]),
                                     conditions_list[0])

    def parse(self, request):
        filter_terms_with_values = [
            (filter_term, value) for filter_term, value in request.GET.dict().items()
            if not filter_term.startswith('_') and filter_term not in request._rest_context
        ]
        if not filter_terms_with_values:
            return None

        conditions_list = []
        for filter_term, value in filter_terms_with_values:
            identifiers = filter_term.split(LOOKUP_SEP)

            if len(identifiers) > 1 and identifiers[-1].upper() == LOGICAL_OPERATORS.NOT:
                identifiers = identifiers[:-1]
                exclude = True
            else:
                exclude = False

            if len(identifiers) > 1 and identifiers[-1].lower() in OPERATORS:
                operator_slug = identifiers[-1].lower()
                identifiers = identifiers[:-1]
            else:
                operator_slug = OPERATORS.EQ

            try:
                cleaned_value = self._clean_value(operator_slug, value)
            except FilterParserError as ex:
                raise FilterParserError(
                    'Invalid filter term {}. {}'.format('{}={}'.format(filter_term, value), ex)
                )

            condition = Term(operator_slug, identifiers, cleaned_value, '{}={}'.format(filter_term, value))
            if exclude:
                condition = ComposedCondition(LOGICAL_OPERATORS.NOT, condition)
            conditions_list.append(condition)
        return self._parse_to_composed_conditions(conditions_list)



class ResourceMetamodel:

    def __init__(self, resource):
        self.resource = resource

    def _request(self, obj):
        form = self.resource._get_form(inst=obj, initial=self.resource._get_form_initial(obj))
        result = {}
        for key, field in form.fields.items():

            choices = getattr(field, 'choices', None)
            if choices:
                choices = list(choices)

            result[key] = {
                'type': field.__class__.__name__,
                'is_required': field.required,
                'verbose_name': field.label,
                'initial': form[key].value(),
                'widget': field.widget.__class__.__name__,
                'help_text': field.help_text,
                'choices': choices,
            }
        return result

    def _get_resource_method_fields(self, fields):
        out = dict()
        # TODO
        for field in fields.flat():
            t = getattr(self.resource, str(field), None)
            if t and callable(t):
                out[field] = t
        return out

    def _get_model_fields(self):
        out = dict()
        for f in self.resource.model._meta.fields:
            out[f.name] = f
        return out

    def _get_m2m_fields(self):
        out = dict()
        for mf in self.resource.model._meta.many_to_many:
            if mf.serialize:
                out[mf.name] = mf
        return out

    def _response(self, obj):
        result = {}

        fields = self.resource.get_fields(obj)

        resource_method_fields = self._get_resource_method_fields(fields)
        model_fields = self._get_model_fields()
        m2m_fields = self._get_m2m_fields()
        for field_name in fields.flat():
            if field_name in resource_method_fields:
                result[field_name] = {
                    'type': 'method',
                    'verbose_name': getattr(resource_method_fields[field_name], 'short_description', None),
                }
            elif field_name in m2m_fields:
                result[field_name] = {
                    'type': m2m_fields[field_name].__class__.__name__,
                    'verbose_name': m2m_fields[field_name].verbose_name,
                }
            elif field_name in model_fields:
                result[field_name] = {
                    'type': model_fields[field_name].__class__.__name__,
                    'verbose_name': model_fields[field_name].verbose_name,
                }
            else:
                deskriptor = getattr(self.resource.model, field_name, None)
                if deskriptor and hasattr(deskriptor, 'related'):
                    result[field_name] = {
                        'type': deskriptor.__class__.__name__,
                    }
                else:
                    result[field_name] = {
                        'type': 'method',
                    }

        return result

    def get(self, obj):
        return {
            'RESPONSE': self._response(obj)
        }

    def post(self, obj):
        return {
            'REQUEST': self._request(obj),
            'RESPONSE': self._response(obj)
        }

    def put(self, obj):
        return {
            'REQUEST': self._request(obj),
            'RESPONSE': self._response(obj)
        }


from django.core.cache import cache


class DefaultRESTCache:
    """
    Cache for improve REST efficiency, works only for GET method
    """

    def _get_cache(self):
        return cache

    def _get_key(self, request):
        return request.get_full_path()

    def cache_response(self, request, response):
        rm = request.method.upper()
        if rm == 'GET':
            self._cache_response(request, response)

    def _cache_response(self, request, response):
        self._get_cache().set(self._get_key(request), response)

    def get_response(self, request):
        rm = request.method.upper()
        if rm == 'GET':
            return self._get_response(request)

    def _get_response(self, request):
        return self._get_cache().get(self._get_key(request))



from django.utils.translation import ugettext
from django.db.models.query import QuerySet

from .exception import RESTException


class BasePaginator:

    @property
    def page_qs(self):
        raise NotImplementedError

    @property
    def headers(self):
        return {}


class BaseOffsetPaginator(BasePaginator):
    """
    REST paginator for list and querysets
    """

    MAX_BIG_INT = pow(2, 63) - 1

    def __init__(self, qs, request):
        self.qs = qs
        self.offset = self._get_offset(request)
        self.base = self._get_base(request)
        self.total = self._get_total()
        self.next_offset = self._get_next_offset()
        self.prev_offset = self._get_prev_offset()

    def _get_next_offset(self):
        return self.offset + self.base if self.base and self.offset + self.base < self.total else None

    def _get_prev_offset(self):
        return None if self.offset == 0 or not self.base else max(self.offset - self.base, 0)

    def _get_total(self):
        if isinstance(self.qs, QuerySet):
            return self.qs.count()
        else:
            return len(self.qs)

    def _get_offset(self, request):
        offset = request._rest_context.get('offset', '0')
        if offset.isdigit():
            offset_int = int(offset)
            if offset_int > self.MAX_BIG_INT:
                raise RESTException(ugettext('Offset must be lower or equal to {}').format(self.MAX_BIG_INT))
            else:
                return offset_int
        else:
            raise RESTException(ugettext('Offset must be natural number'))

    def _get_base(self, request):
        base = request._rest_context.get('base')
        if not base:
            return None
        elif base.isdigit():
            base_int = int(base)
            if base_int > self.MAX_BIG_INT:
                raise RESTException(ugettext('Base must lower or equal to {}').format(self.MAX_BIG_INT))
            else:
                return base_int
        else:
            raise RESTException(ugettext('Base must be natural number or empty'))

    @property
    def page_qs(self):
        if self.base is not None:
            return self.qs[self.offset:(self.offset + self.base)]
        else:
            return self.qs[self.offset:]

    @property
    def headers(self):
        return {
            k: v for k, v in {
                'X-Total': self.total,
                'X-Next-Offset': self.next_offset,
                'X-Prev-Offset': self.prev_offset,
            }.items() if v is not None
        }


class BaseOffsetPaginatorWithoutTotal(BaseOffsetPaginator):

    def _get_total(self):
        return None

    def _get_next_offset(self):
        if not self.base:
            return None
        next_offset = self.offset + self.base
        return next_offset if self.qs[next_offset:next_offset + 1].exists() else None



from datetime import datetime

from germanium.tools.trivials import assert_equal
from germanium.tools.http import assert_http_bad_request, build_url
from germanium.tools.rest import assert_valid_JSON_response

from .factories import UserFactory, IssueFactory
from .test_case import PystonTestCase


class FilterTestCase(PystonTestCase):

    def test_override_extra_filter_fields(self):
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, filter='created_at__gt="1.1.1980"')))
        assert_valid_JSON_response(
            self.get(build_url(self.USER_API_URL, filter='email contains "test@test.cz"'))
        )

    def test_invalid_filter_format(self):
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, filter='email="test@test.cz" AND')))
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, filter='invalid')))
        assert_http_bad_request(self.get(build_url(self.USER_API_URL,
                                                   filter='email="test@test.cz" AND OR email="test@test.cz"')))

    def test_issue_can_filter_only_with_readable_fields_and_extra_field(self):
        assert_valid_JSON_response(
            self.get(build_url(self.ISSUE_API_URL, filter='solver__created_at="1.1.2017"'))
        )
        assert_valid_JSON_response(self.get(build_url(self.ISSUE_API_URL, filter='created_at>"1.1.2017"')))
        assert_http_bad_request(
            self.get(build_url(self.ISSUE_API_URL, filter='solver__manual_created_date>"1.1.2017"'))
        )

    def test_filter_issue_by_datetime(self):
        now = datetime.now()
        [IssueFactory() for _ in range(5)]

        resp = self.get(build_url(self.ISSUE_API_URL))
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(build_url(self.ISSUE_API_URL, filter='created_at>"{}"'.format(now.isoformat())))
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(build_url(self.ISSUE_API_URL, filter='created_at="{}"'.format(now.isoformat())))
        assert_equal(len(self.deserialize(resp)), 0)

        resp = self.get(build_url(self.ISSUE_API_URL, filter='created_at<"{}"'.format(now.isoformat())))
        assert_equal(len(self.deserialize(resp)), 0)

        resp = self.get(build_url(self.ISSUE_API_URL, filter='created_at contains "{}"'.format(now.date())))
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(build_url(self.ISSUE_API_URL, filter='created_at contains "{}"'.format(now.year)))
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(build_url(self.ISSUE_API_URL,
                                  filter='created_at contains "{}"'.format('{} {}'.format(now.month, now.year))))
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(build_url(self.ISSUE_API_URL,
                                  filter='created_at contains "{}"'.format('{} {}'.format(now.month + 1, now.year))))
        assert_equal(len(self.deserialize(resp)), 0)

        resp = self.get(build_url(self.ISSUE_API_URL, filter='created_at__day = "{}"'.format(now.day)))
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(
            build_url(
                self.ISSUE_API_URL,
                filter='created_at__day = "{}" AND created_at__year = "{}"'.format(now.day, now.year)
            )
        )
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(
            build_url(
                self.ISSUE_API_URL,
                filter='created_at__day = "{}" AND created_at__year = "{}"'.format(now.day + 1, now.year)
            )
        )
        assert_equal(len(self.deserialize(resp)), 0)

        resp = self.get(
            build_url(
                self.ISSUE_API_URL,
                filter='created_at__day = "{}" OR created_at__year = "{}"'.format(now.day + 1, now.year)
            )
        )
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(
            build_url(
                self.ISSUE_API_URL,
                filter='(created_at__day = "{}" AND created_at__year = "{}") OR (created_at > "{}")'.format(
                    now.day + 1, now.year, now.isoformat()
                )
            )
        )
        assert_equal(len(self.deserialize(resp)), 5)

    def test_filter_issue_by_datetime_querystring_filter_parser(self):
        now = datetime.now()
        [IssueFactory() for _ in range(5)]
        resp = self.get(
            build_url(
                self.ISSUE_API_URL, created_at__day=now.day, created_at__year=now.year
            )
        )
        assert_equal(len(self.deserialize(resp)), 5)

        resp = self.get(
            build_url(
                self.ISSUE_API_URL, created_at__day=now.day + 1, created_at__year=now.year
            )
        )
        assert_equal(len(self.deserialize(resp)), 0)

    def test_boolean_filter(self):
        user = UserFactory(is_superuser=False)
        superuser = UserFactory(is_superuser=True)

        data = self.deserialize(self.get(build_url( self.USER_API_URL, is_superuser=0)))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user.pk)

        data = self.deserialize(self.get(build_url(self.USER_API_URL, is_superuser=1)))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], superuser.pk)

        data = self.deserialize(self.get(build_url( self.USER_API_URL, is_superuser__not=0)))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], superuser.pk)

        data = self.deserialize(self.get(build_url( self.USER_API_URL, is_superuser__gt=0)))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], superuser.pk)

        data = self.deserialize(self.get(build_url( self.USER_API_URL, is_superuser__lt=0)))
        assert_equal(len(data), 0)

        data = self.deserialize(self.get(build_url( self.USER_API_URL, is_superuser__lt=1)))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user.pk)

        assert_http_bad_request(self.get(build_url(self.USER_API_URL, is_superuser='invalid')))
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, is_superuser=3)))
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, is_superuser__gt=3)))
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, is_superuser__not='invalid')))
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, is_superuser__not='__none__')))

    def test_string_filter(self):
        issue1 = IssueFactory(name='issue with text: protocol')
        issue2 = IssueFactory(name='issue with text: alcohol')

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='name = "issue with text: protocol"')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='name != "issue with text: protocol"')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue2.pk)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='name contains "protocol"')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='name icontains "Protocol"')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='name startswith "issue"')))
        assert_equal(len(data), 2)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='name iendswith "oL"')))
        assert_equal(len(data), 2)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL, filter='name in ["issue with text: protocol", "issue with text: alcohol"]'
                )
            )
        )
        assert_equal(len(data), 2)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL, filter='name in ["issue with text: protocol", "issue with text"]'
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='name iexact "issue With tExt: prOtocol"')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

    def test_integer_filter(self):
        issue1 = IssueFactory(logged_minutes=150)
        issue2 = IssueFactory(logged_minutes=31)
        issue3 = IssueFactory(logged_minutes=None)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='logged_minutes = 150')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='logged_minutes >= 31')))
        assert_equal(len(data), 2)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='logged_minutes <= 150')))
        assert_equal(len(data), 2)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='logged_minutes < 50')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue2.pk)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='logged_minutes in [null, 150, 31]')))
        assert_equal(len(data), 3)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, filter='logged_minutes = null')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue3.pk)

    def test_foreign_key_filter(self):
        issue1 = IssueFactory()
        issue2 = IssueFactory(solver=issue1.created_by)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, created_by=issue1.created_by.pk)))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, created_by__not=issue1.created_by.pk)))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue2.pk)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    created_by__in='[{}, {}]'.format(issue1.created_by.pk, issue2.created_by.pk)
                )
            )
        )
        assert_equal(len(data), 2)

        data = self.deserialize(self.get(build_url(self.ISSUE_API_URL, solver='__none__')))
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        assert_http_bad_request(self.get(build_url(self.ISSUE_API_URL, solver='invalid')))

    def test_many_to_many_filter(self):
        user1 = UserFactory()
        user2 = UserFactory()

        issue1 = IssueFactory()
        issue2 = IssueFactory()

        issue1.watched_by.add(user1, user2)
        issue2.watched_by.add(user1)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    watched_by=user2.pk
                )
            )
        )
        assert_equal(len(data), 1)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    watched_by__in='({},{})'.format(user1.pk, user2.pk)
                )
            )
        )
        assert_equal(len(data), 2)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    watched_by__in='({})'.format(user1.pk)
                )
            )
        )
        assert_equal(len(data), 2)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    watched_by__in='({})'.format(user2.pk)
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    watched_by__all='({},{})'.format(user1.pk, user2.pk)
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    watched_by__all='({})'.format(user1.pk)
                )
            )
        )
        assert_equal(len(data), 2)

    def test_many_to_one_filter(self):
        issue1 = IssueFactory()
        issue2 = IssueFactory()
        issue3 = IssueFactory(created_by=issue1.created_by)

        user1 = issue1.created_by

        assert_http_bad_request(self.get(build_url(self.USER_API_URL, created_issues=user1)))

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    created_issues__in='({},{})'.format(issue1.pk, issue3.pk)
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user1.pk)

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    created_issues__in='({})'.format(issue1.pk)
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user1.pk)

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    created_issues__in='({},{},{})'.format(issue1.pk, issue2.pk, issue3.pk)
                )
            )
        )
        assert_equal(len(data), 2)

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    created_issues__all='({},{},{})'.format(issue1.pk, issue2.pk, issue3.pk)
                )
            )
        )
        assert_equal(len(data), 0)

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    created_issues__all='({},{})'.format(issue1.pk, issue3.pk)
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user1.pk)

    def test_custom_field_filter(self):
        user = UserFactory(email='test1@email.com')

        assert_http_bad_request(self.get(build_url(self.USER_API_URL, email='test1@email.com')))
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, email__icontains='test1')))
        assert_http_bad_request(self.get(build_url(self.USER_API_URL, email__not='test1')))

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    email__contains='test'
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user.pk)

    def test_custom_method_filter(self):
        user1 = UserFactory()
        user2 = UserFactory()

        issue1 = IssueFactory()
        issue2 = IssueFactory()

        issue1.watched_by.add(user1, user2)
        issue2.watched_by.add(user1)

        assert_http_bad_request(self.get(build_url(self.USER_API_URL, watched_issues_count='invalid')))

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    watched_issues_count=2
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user1.pk)

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    watched_issues_count=1
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user2.pk)

    def test_filter_by_decorator(self):
        issue1 = IssueFactory(description='test1')
        issue2 = IssueFactory(description='test2')

        assert_http_bad_request(self.get(build_url(self.ISSUE_API_URL, description='test1')))

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    short_description__contains='test'
                )
            )
        )
        assert_equal(len(data), 2)

        data = self.deserialize(
            self.get(
                build_url(
                    self.ISSUE_API_URL,
                    short_description='test1'
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], issue1.pk)

    def test_resource_filter(self):
        user1 = UserFactory()
        user2 = UserFactory()

        IssueFactory(solver=user1, logged_minutes=120, estimate_minutes=60)
        IssueFactory(solver=user2, logged_minutes=59, estimate_minutes=60)

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    issues__overtime=1
                )
            )
        )
        assert_equal(len(data), 1)
        assert_equal(data[0]['id'], user1.pk)

        data = self.deserialize(
            self.get(
                build_url(
                    self.USER_API_URL,
                    issues__overtime=0,
                )
            )
        )
        assert_equal(len(data), 5)



from six.moves.urllib.parse import urlencode

from django.test import TestCase
from django.test.utils import override_settings

from germanium.test_cases.rest import RESTTestCase
from germanium.tools.trivials import assert_true, assert_false, assert_equal
from germanium.anotations import data_provider

from pyston.resource import (ACCESS_CONTROL_ALLOW_ORIGIN, ACCESS_CONTROL_EXPOSE_HEADERS,
                             ACCESS_CONTROL_ALLOW_CREDENTIALS, ACCESS_CONTROL_ALLOW_HEADERS,
                             ACCESS_CONTROL_ALLOW_METHODS, ACCESS_CONTROL_MAX_AGE)

from .test_case import PystonTestCase


FOO_DOMAIN = 'http://foo.pyston.net'
BAR_DOMAIN = 'http://bar.pyston.net'


class CorsTestCase(PystonTestCase):

    @override_settings(PYSTON_CORS=False)
    @data_provider('get_users_data')
    def test_with_turned_off_cors_headers_is_not_included(self, number, data):
        resp = self.options(self.USER_API_URL)
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_ORIGIN))
        assert_false(resp.has_header(ACCESS_CONTROL_EXPOSE_HEADERS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_CREDENTIALS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_HEADERS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_METHODS))
        assert_false(resp.has_header(ACCESS_CONTROL_MAX_AGE))

        resp = self.get(self.USER_API_URL)
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_ORIGIN))
        assert_false(resp.has_header(ACCESS_CONTROL_EXPOSE_HEADERS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_CREDENTIALS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_HEADERS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_METHODS))
        assert_false(resp.has_header(ACCESS_CONTROL_MAX_AGE))

    @override_settings(PYSTON_CORS=True, PYSTON_CORS_WHITELIST=[FOO_DOMAIN[7:]])
    @data_provider('get_users_data')
    def test_option_with_turned_on_cors_headers_is_included_with_valid_origin(self, number, data):
        resp = self.options(self.USER_API_URL)
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_ORIGIN))
        assert_true(resp.has_header(ACCESS_CONTROL_EXPOSE_HEADERS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_CREDENTIALS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_HEADERS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_METHODS))
        assert_true(resp.has_header(ACCESS_CONTROL_MAX_AGE))

        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': FOO_DOMAIN})
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_ORIGIN))
        assert_true(resp.has_header(ACCESS_CONTROL_EXPOSE_HEADERS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_CREDENTIALS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_HEADERS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_METHODS))
        assert_true(resp.has_header(ACCESS_CONTROL_MAX_AGE))

        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': BAR_DOMAIN})
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_ORIGIN))
        assert_true(resp.has_header(ACCESS_CONTROL_EXPOSE_HEADERS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_CREDENTIALS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_HEADERS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_METHODS))

    @override_settings(PYSTON_CORS=True, PYSTON_CORS_WHITELIST=[FOO_DOMAIN[7:]])
    @data_provider('get_users_data')
    def test_with_turned_on_cors_headers_is_included_with_valid_origin(self, number, data):
        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': FOO_DOMAIN})
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_ORIGIN))
        assert_true(resp.has_header(ACCESS_CONTROL_EXPOSE_HEADERS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_CREDENTIALS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_HEADERS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_METHODS))
        assert_true(resp.has_header(ACCESS_CONTROL_MAX_AGE))

        resp = self.get(self.USER_API_URL, headers={'HTTP_ORIGIN': FOO_DOMAIN})
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_ORIGIN))
        assert_true(resp.has_header(ACCESS_CONTROL_EXPOSE_HEADERS))
        assert_true(resp.has_header(ACCESS_CONTROL_ALLOW_CREDENTIALS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_HEADERS))
        assert_false(resp.has_header(ACCESS_CONTROL_ALLOW_METHODS))
        assert_true(resp.has_header(ACCESS_CONTROL_MAX_AGE))

    @override_settings(PYSTON_CORS=True, PYSTON_CORS_MAX_AGE=60 * 10)
    @data_provider('get_users_data')
    def test_cors_max_age(self, number, data):
        resp = self.options(self.USER_API_URL)
        assert_equal(resp[ACCESS_CONTROL_MAX_AGE], '600')

    @override_settings(PYSTON_CORS=True, PYSTON_CORS_ALLOW_CREDENTIALS=True)
    @data_provider('get_users_data')
    def test_cors_allow_credentials_true(self, number, data):
        resp = self.options(self.USER_API_URL)
        assert_equal(resp[ACCESS_CONTROL_ALLOW_CREDENTIALS], 'true')

    @override_settings(PYSTON_CORS=True, PYSTON_CORS_ALLOW_CREDENTIALS=False)
    @data_provider('get_users_data')
    def test_cors_allow_credentials_false(self, number, data):
        resp = self.options(self.USER_API_URL)
        assert_equal(resp[ACCESS_CONTROL_ALLOW_CREDENTIALS], 'false')

    @override_settings(PYSTON_CORS=True, PYSTON_CORS_ALLOW_CREDENTIALS=False, PYSTON_CORS_WHITELIST=[FOO_DOMAIN[7:]])
    @data_provider('get_users_data')
    def test_cors_allow_headers(self, number, data):
        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': FOO_DOMAIN})
        assert_equal(resp[ACCESS_CONTROL_ALLOW_HEADERS],
                          ', '.join(('X-Base', 'X-Offset', 'X-Fields', 'Origin', 'Content-Type', 'Accept')))

        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': BAR_DOMAIN})
        assert_equal(resp[ACCESS_CONTROL_ALLOW_HEADERS],
                          ', '.join(('X-Base', 'X-Offset', 'X-Fields', 'Origin', 'Content-Type', 'Accept')))

        resp = self.options(self.USER_API_URL)
        assert_false(ACCESS_CONTROL_ALLOW_HEADERS in resp)

    @override_settings(PYSTON_CORS=True, PYSTON_CORS_ALLOW_CREDENTIALS=False)
    @data_provider('get_users_data')
    def test_cors_allow_exposed_headers(self, number, data):
        resp = self.options(self.USER_API_URL)
        assert_equal(resp[ACCESS_CONTROL_EXPOSE_HEADERS],
                          ', '.join(('X-Total', 'X-Serialization-Format-Options', 'X-Fields-Options')))

    @override_settings(PYSTON_CORS=True, PYSTON_CORS_ALLOW_CREDENTIALS=False)
    @data_provider('get_users_data')
    def test_cors_allow_methods(self, number, data):
        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': FOO_DOMAIN})
        assert_equal(set(resp[ACCESS_CONTROL_ALLOW_METHODS].split(', ')), {'OPTIONS'})

        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': FOO_DOMAIN,
                                                        'HTTP_ACCESS_CONTROL_REQUEST_METHOD': 'GET'})
        assert_equal(set(resp[ACCESS_CONTROL_ALLOW_METHODS].split(', ')), {'GET'})

        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': BAR_DOMAIN})
        assert_equal(set(resp[ACCESS_CONTROL_ALLOW_METHODS].split(', ')), {'OPTIONS'})

        resp = self.options(self.USER_API_URL, headers={'HTTP_ORIGIN': BAR_DOMAIN,
                                                        'HTTP_ACCESS_CONTROL_REQUEST_METHOD': 'POST'})
        assert_equal(set(resp[ACCESS_CONTROL_ALLOW_METHODS].split(', ')), {'POST'})

        resp = self.options(self.USER_API_URL)
        assert_false(ACCESS_CONTROL_ALLOW_METHODS in resp)



from __future__ import unicode_literals

import json
import xml.dom.minidom

from germanium.tools import assert_true, assert_equal

from unittest.case import TestCase

from app.models import User

from pyston.serializer import serialize


class DirectSerializationTestCase(TestCase):

    def test_serialization(self):
        for i in range(10):
            User.objects.create(is_superuser=True, email='test{}@test.cz'.format(i))
        assert_true(isinstance(json.loads((serialize(User.objects.all()))), list))
        assert_true(isinstance(json.loads((serialize(User.objects.first()))), dict))

        assert_equal(
            set(json.loads((serialize(User.objects.first()))).keys()),
            {'id', 'created_at', '_obj_name', 'email', 'contract', 'solving_issue', 'first_name', 'last_name',
             'manual_created_date'}
        )
        assert_equal(
            set(json.loads((serialize(User.objects.first(), ('id',)))).keys()),
            {'id'}
        )

        xml.dom.minidom.parseString(serialize(User.objects.first(), converter_name='xml'))



from decimal import Decimal, InvalidOperation

from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _
from django.db.models import Q
from django.core.validators import validate_ipv4_address, validate_ipv46_address
from django.core.exceptions import ValidationError

from chamber.utils.datetimes import make_aware
from chamber.utils.datastructures import Enum

from dateutil.parser import DEFAULTPARSER

from pyston.utils import LOOKUP_SEP

from .exceptions import FilterValueError, OperatorFilterError


OPERATORS = Enum(
    ('GT', 'gt'),
    ('LT', 'lt'),
    ('EQ', 'eq'),
    ('NEQ', 'neq'),
    ('LTE', 'lte'),
    ('GTE', 'gte'),
    ('CONTAINS', 'contains'),
    ('ICONTAINS', 'icontains'),
    ('RANGE', 'range'),
    ('EXACT', 'exact'),
    ('IEXACT', 'iexact'),
    ('STARTSWITH', 'startswith'),
    ('ISTARTSWITH', 'istartswith'),
    ('ENDSWITH', 'endswith'),
    ('IENDSWITH', 'iendswith'),
    ('IN', 'in'),
    ('RANGE', 'range'),
    ('ALL', 'all'),
)


NONE_LABEL = _('(None)')


class Operator:
    """
    Operator is used for specific type of filters that allows more different ways how to filter queryset data according
    to input operator between identifier and value.
    """

    def get_q(self, value, request):
        """
        Method must be implemented inside descendant and should return django db Q object that will be used for purpose
        of filtering queryset.
        """
        raise NotImplementedError


class EqualOperator(Operator):
    """
    Equal operator returns Q object that filter data if cleaned value is equal to DB object.
    """

    def get_q(self, filter, value, operator_slug, request):
        return Q(**{filter.get_full_filter_key(): filter.clean_value(value, operator_slug, request)})


class NotEqualOperator(Operator):
    """
    Not equal operator is opposite to equal operator.
    """

    def get_q(self, filter, value, operator_slug, request):
        return ~Q(**{filter.get_full_filter_key(): filter.clean_value(value, operator_slug, request)})


class SimpleOperator(Operator):
    """
    Simple operator is used for more operators that differ (in a DB view) only with string ORM operator.
    """

    def __init__(self, orm_operator):
        self.orm_operator = orm_operator

    def get_q(self, filter, value, operator_slug, request):
        value = filter.clean_value(value, operator_slug, request)
        return Q(**{'{}__{}'.format(filter.get_full_filter_key(), self.orm_operator): value})


class ListOperatorMixin:
    """
    The mixin is helper for operator objects that accepts list of data.
    It adds clean method that cleans list of values and returns an error to the concrete item of the list.
    """

    def _clean_list_values(self, filter, operator_slug, request, values):
        cleaned_values = []
        i = 0
        for value in values:
            try:
                cleaned_values.append(filter.clean_value(value, operator_slug, request))
            except FilterValueError as ex:
                raise FilterValueError(ugettext('Invalid value inside list at position {}, {}').format(i, ex))
            i += 1
        return cleaned_values


class SimpleListOperator(ListOperatorMixin, Operator):
    """
    The operator object is alternative to the Simple operator. It only accepts data in list format.
    """

    def __init__(self, orm_operator):
        self.orm_operator = orm_operator

    def get_q(self, filter, values, operator_slug, request):
        if not isinstance(values, list):
            raise FilterValueError(ugettext('Value must be list'))
        else:
            values = self._clean_list_values(filter, operator_slug, request, values)
        q = Q(**{
            '{}__{}'.format(filter.get_full_filter_key(), self.orm_operator): {v for v in values if values is not None}
        })
        if None in values:
            q |= Q(**{'{}__isnull'.format(filter.get_full_filter_key()): True})
        return q


class RangeOperator(ListOperatorMixin, Operator):
    """
    Operator filters range between two input values.
    """

    def get_q(self, filter, values, operator_slug, request):
        if not isinstance(values, list) or not len(values) != 2:
            raise FilterValueError(ugettext('Value must be list with two values'))
        else:
            values = self._clean_list_values(filter, operator_slug, request, values)
        return Q(**{'{}__range'.format(filter.get_full_filter_key()): values})


class AllListOperator(ListOperatorMixin, Operator):
    """
    Operator that is used for filtering m2m or m2o relations. All sent values must be related.
    """

    def get_q(self, filter, values, operator_slug, request):
        if not isinstance(values, list):
            raise FilterValueError(ugettext('Value must be list'))
        else:
            values = self._clean_list_values(filter, operator_slug, request, values)

        qs_obj_with_all_values = filter.field.model.objects.all()
        for v in set(values):
            qs_obj_with_all_values = qs_obj_with_all_values.filter(**{filter.identifiers[-1]: v})
        return Q(
            **{
                '{}__in'.format(
                    LOOKUP_SEP.join(filter.identifiers[:-1] + ['pk'])
                ): qs_obj_with_all_values.values('pk')
            }
        )


class DateContainsOperator(Operator):
    """
    Specific operator for datetime that allows filter date or datetime that is not fully sent.
    For example filter according to month and year (1.2017).
    """

    def get_q(self, filter, value, operator_slug, request):
        filter_term = {}
        value = filter.clean_value(value, operator_slug, request)
        for attr in filter.get_suffixes():
            date_val = getattr(value, attr)
            if date_val:
                filter_term[LOOKUP_SEP.join((filter.get_full_filter_key(), attr))] = date_val
        return Q(**filter_term)


EQ = EqualOperator()
NEQ = NotEqualOperator()
LT = SimpleOperator('lt')
GT = SimpleOperator('gt')
LTE = SimpleOperator('lte')
GTE = SimpleOperator('gte')
CONTAINS = SimpleOperator('contains')
ICONTAINS = SimpleOperator('icontains')
EXACT = SimpleOperator('exact')
IEXACT = SimpleOperator('iexact')
STARTSWITH = SimpleOperator('startswith')
ISTARTSWITH = SimpleOperator('istartswith')
ENDSWITH = SimpleOperator('endswith')
IENDSWITH = SimpleOperator('iendswith')
IN = SimpleListOperator('in')
RANGE = RangeOperator()
ALL = AllListOperator()
DATE_CONTAINS = DateContainsOperator()
PK_CONTAINS = SimpleOperator('pk__contains')
PK_ICONTAINS = SimpleOperator('pk__icontains')


class Filter:
    """
    Filters purpose is return Q object that is used for filtering data that resource returns.
    Filter can be joined to the field, method or resource.
    """

    suffixes = {}
    choices = None

    def __init__(self, identifiers_prefix, identifiers, identifiers_suffix, model, field=None, method=None):
        """
        Filter init values are these:
        :param identifiers_prefix: because filters are recursive if model relations property contains list of
               identifiers that was used for recursive searching the filter.
        :param identifiers: list of identifiers that conclusively identifies the filter.
        :param identifiers_suffix: list of suffixes that can be used for more specific filters.
               For example for a date filter can be used suffixes month, day, year.
        :param model: Django model class of filtered object.
        :param field: model field which is related with filter.
        :param method: method that is related with filter.
        method and field cannot be set together.
        """
        assert field is None or method is None, 'Field and method cannot be set together'

        self.identifiers_prefix = identifiers_prefix
        self.identifiers = identifiers
        self.identifiers_suffix = identifiers_suffix
        self.full_identifiers = identifiers_prefix + identifiers + identifiers_suffix
        self.field = field
        self.method = method
        self.model = model

    def get_allowed_operators(self):
        """
        :return: list of allowed operators to the concrete filter.
        """
        return self.allowed_operators

    @classmethod
    def get_suffixes(cls):
        """
        :return: set of allowed suffixes for the operator.
        """
        return cls.suffixes

    def get_q(self, value, operator_slug, request):
        """
        Method must be implemented inside descendant and should return django db Q object that will be used for purpose
        of filtering rest response data.
        """
        raise NotImplementedError


class OperatorsFilterMixin:
    """
    Mixin is used for specific type of filter that uses operator objects.
    """

    operators = ()

    def clean_value(self, value, operator_slug, request):
        """
        Method that cleans input value to the filter specific format.
        """
        return value

    def get_allowed_operators(self):
        return [operator_key for operator_key, operator in self.operators]

    def get_operator_obj(self, operator_slug):
        """
        :return: concrete operator object for the specific operator key.
        """
        operator_obj = dict(self.operators).get(operator_slug)
        if not operator_obj:
            raise OperatorFilterError
        return operator_obj

    def get_full_filter_key(self):
        return LOOKUP_SEP.join(self.full_identifiers)

    def get_q(self, value, operator_slug, request):
        operator_obj = self.get_operator_obj(operator_slug)
        return operator_obj.get_q(self, value, operator_slug, request)


class MethodFilter(Filter):
    """
    Abstract parent for all method filters.
    """

    def __init__(self, identifiers_prefix, identifiers, identifiers_suffix, model, method=None):
        assert method, 'Method is required'
        super(MethodFilter, self).__init__(identifiers_prefix, identifiers, identifiers_suffix, model, method=method)


class ModelFieldFilter(Filter):
    """
    Abstract parent for all model field filters.
    """

    def __init__(self, identifiers_prefix, identifiers, identifiers_suffix, model, field=None):
        assert field, 'Field is required'
        super(ModelFieldFilter, self).__init__(identifiers_prefix, identifiers, identifiers_suffix, model, field=field)


class OperatorsModelFieldFilter(OperatorsFilterMixin, ModelFieldFilter):
    pass


class BooleanFilterMixin:
    """
    Helper that contains cleaner for boolean input values.
    """

    choices = (
        (1, _('Yes')),
        (0, _('No'))
    )

    def clean_value(self, value, operator_slug, request):
        if isinstance(value, bool):
            return value
        elif value in {'0', '1'}:
            return value == '1'
        else:
            raise FilterValueError(ugettext('Value must be boolean'))


class NullBooleanFilterMixin(BooleanFilterMixin):
    """
    Helper that contains cleaner for boolean input values where value can be None.
    """

    choices = (
        (None, NONE_LABEL),
        (1, _('Yes')),
        (0, _('No'))
    )

    def clean_value(self, value, operator_slug, request):
        if value is None:
            return value
        else:
            return super(NullBooleanFilterMixin, self).clean_value(value, operator_slug, request)


class BooleanFieldFilter(BooleanFilterMixin, OperatorsModelFieldFilter):

    operators = (
        (OPERATORS.EQ, EQ),
        (OPERATORS.NEQ, NEQ),
        (OPERATORS.LT, LT),
        (OPERATORS.GT, GT),
    )


class NullBooleanFieldFilter(NullBooleanFilterMixin, OperatorsModelFieldFilter):

    operators = (
        (OPERATORS.EQ, EQ),
        (OPERATORS.NEQ, NEQ),
        (OPERATORS.LT, LT),
        (OPERATORS.GT, GT),
    )


class NumberFieldFilter(OperatorsModelFieldFilter):

    operators = (
        (OPERATORS.EQ, EQ),
        (OPERATORS.NEQ, NEQ),
        (OPERATORS.LT, LT),
        (OPERATORS.GT, GT),
        (OPERATORS.LTE, LTE),
        (OPERATORS.GTE, GTE),
        (OPERATORS.IN, IN),
    )


class IntegerFieldFilterMixin:
    """
    Helper that contains cleaner for integer input values.
    """

    def clean_value(self, value, operator_slug, request):
        if value is None:
            return value
        try:
            return int(value)
        except (ValueError, TypeError):
            raise FilterValueError(ugettext('Value must be integer'))


class IntegerFieldFilter(IntegerFieldFilterMixin, NumberFieldFilter):
    pass


class FloatFieldFilter(NumberFieldFilter):

    def clean_value(self, value, operator_slug, request):
        if value is None:
            return value
        try:
            return float(value)
        except (ValueError, TypeError):
            raise FilterValueError(ugettext('Value must be float'))


class DecimalFieldFilter(NumberFieldFilter):

    def clean_value(self, value, operator_slug, request):
        if value is None:
            return value
        try:
            return Decimal(value)
        except InvalidOperation:
            raise FilterValueError(ugettext('Value must be decimal'))


class StringFieldFilter(OperatorsModelFieldFilter):

    operators = (
        (OPERATORS.ICONTAINS, ICONTAINS),
        (OPERATORS.CONTAINS, CONTAINS),
        (OPERATORS.EXACT, EXACT),
        (OPERATORS.IEXACT, IEXACT),
        (OPERATORS.STARTSWITH, STARTSWITH),
        (OPERATORS.ISTARTSWITH, ISTARTSWITH),
        (OPERATORS.ENDSWITH, ENDSWITH),
        (OPERATORS.IENDSWITH, IENDSWITH),
        (OPERATORS.EQ, EQ),
        (OPERATORS.NEQ, NEQ),
        (OPERATORS.LT, LT),
        (OPERATORS.GT, GT),
        (OPERATORS.LTE, LTE),
        (OPERATORS.GTE, GTE),
        (OPERATORS.IN, IN),
    )


class CaseSensitiveStringFieldFilter(StringFieldFilter):

    operators = (
        (OPERATORS.CONTAINS, CONTAINS),
        (OPERATORS.EXACT, EXACT),
        (OPERATORS.STARTSWITH, STARTSWITH),
        (OPERATORS.ENDSWITH, ENDSWITH),
        (OPERATORS.EQ, EQ),
        (OPERATORS.NEQ, NEQ),
        (OPERATORS.LT, LT),
        (OPERATORS.GT, GT),
        (OPERATORS.LTE, LTE),
        (OPERATORS.GTE, GTE),
        (OPERATORS.IN, IN),
    )


class IPAddressFilterFilter(CaseSensitiveStringFieldFilter):

    def clean_value(self, value, operator_slug, request):
        if value is None:
            return value
        elif operator_slug not in {OPERATORS.CONTAINS, OPERATORS.EXACT, OPERATORS.STARTSWITH, OPERATORS.ENDSWITH}:
            try:
                validate_ipv4_address(value)
            except ValidationError:
                raise FilterValueError(ugettext('Value must be in format IPv4.'))
        return value


class GenericIPAddressFieldFilter(CaseSensitiveStringFieldFilter):

    def clean_value(self, value, operator_slug, request):
        if value is None:
            return value
        elif operator_slug not in {OPERATORS.CONTAINS, OPERATORS.EXACT, OPERATORS.STARTSWITH, OPERATORS.ENDSWITH}:
            try:
                validate_ipv46_address(value)
            except ValidationError:
                raise FilterValueError(ugettext('Value must be in format IPv4 or IPv6.'))
        return value


class DateFilterMixin:

    suffixes = {
        'day', 'month', 'year'
    }

    def _clean_datetime_to_parts(self, value):
        value = DEFAULTPARSER._parse(value, dayfirst='-' not in value)
        value = value[0] if isinstance(value, tuple) else value

        if value is None:
            raise FilterValueError(ugettext('Value cannot be parsed to partial datetime'))
        else:
            return value

    def _clean_integer(self, value):
        try:
            return int(value)
        except ValueError:
            raise FilterValueError(ugettext('Value must be integer'))

    def _clean_datetime(self, value):
        try:
            datetime_value = DEFAULTPARSER.parse(value, dayfirst='-' not in value)
            return make_aware(datetime_value) if datetime_value.tzinfo is None else datetime_value
        except ValueError:
            raise FilterValueError(ugettext('Value must be in format ISO 8601.'))

    def clean_value(self, value, operator_slug, request):
        suffix = self.identifiers_suffix[0] if self.identifiers_suffix else None
        if suffix in self.suffixes:
            return self._clean_integer(value)
        elif operator_slug == OPERATORS.CONTAINS:
            return self._clean_datetime_to_parts(value)
        elif value is None:
            return value
        else:
            return self._clean_datetime(value)


class DateFilter(DateFilterMixin, OperatorsModelFieldFilter):

    operators = (
        (OPERATORS.CONTAINS, DATE_CONTAINS),
        (OPERATORS.EQ, EQ),
        (OPERATORS.NEQ, NEQ),
        (OPERATORS.LT, LT),
        (OPERATORS.GT, GT),
        (OPERATORS.LTE, LTE),
        (OPERATORS.GTE, GTE),
        (OPERATORS.IN, IN),
    )


class DateTimeFilter(DateFilter):

    suffixes = {
        'day', 'month', 'year', 'hour', 'minute', 'second'
    }


class RelatedFieldFilter(OperatorsModelFieldFilter):
    """
    Helper that is used for relation model filters.
    """

    def get_last_rel_field(self, field):
        """
        :return: field that can be used for filtering of the relation filter.
        """
        if not field.is_relation:
            return field
        else:
            next_field = field.related_model._meta.get_field(field.remote_field.field_name)
            return self.get_last_rel_field(next_field)


class ForeignKeyFilter(RelatedFieldFilter):

    operators = (
        (OPERATORS.EQ, EQ),
        (OPERATORS.NEQ, NEQ),
        (OPERATORS.LT, LT),
        (OPERATORS.GT, GT),
        (OPERATORS.LTE, LTE),
        (OPERATORS.GTE, GTE),
        (OPERATORS.IN, IN),
        (OPERATORS.ICONTAINS, ICONTAINS),
        (OPERATORS.CONTAINS, CONTAINS),
    )

    def clean_value(self, value, operator_slug, request):
        if value is None:
            return value
        try:
            return self.get_last_rel_field(self.field).get_prep_value(value)
        except ValueError:
            raise FilterValueError(ugettext('Object with this PK cannot be found'))


class ManyToManyFieldFilter(RelatedFieldFilter):

    operators = (
        (OPERATORS.EQ, EQ),
        (OPERATORS.IN, IN),
        (OPERATORS.ALL, ALL),
        (OPERATORS.CONTAINS, PK_CONTAINS),
        (OPERATORS.ICONTAINS, PK_ICONTAINS),
    )

    def clean_value(self, value, operator_slug, request):
        if value is None or operator_slug in {OPERATORS.CONTAINS, OPERATORS.ICONTAINS}:
            return value
        try:
            return self.get_last_rel_field(
                self.field.related_model._meta.get_field(self.field.m2m_target_field_name())
            ).get_prep_value(value)
        except ValueError:
            raise FilterValueError(ugettext('Object with this PK cannot be found'))


class ForeignObjectRelFilter(RelatedFieldFilter):

    operators = (
        (OPERATORS.EQ, EQ),
        (OPERATORS.IN, IN),
        (OPERATORS.ALL, ALL),
        (OPERATORS.CONTAINS, PK_CONTAINS),
        (OPERATORS.ICONTAINS, PK_ICONTAINS),
    )

    def clean_value(self, value, operator_slug, request):
        if value is None or operator_slug in {OPERATORS.CONTAINS, OPERATORS.ICONTAINS}:
            return value
        try:
            return self.get_last_rel_field(
                self.field.related_model._meta.get_field(self.field.related_model._meta.pk.name)
            ).get_prep_value(value)
        except ValueError:
            raise FilterValueError(ugettext('Object with this PK cannot be found'))


class SimpleFilterMixin:
    """
    Helper that is used for implementation all simple custom filters.
    """

    allowed_operators = None

    def _update_q_with_prefix(self, q):
        """
        Because implementation of custom filter should be as simple as possible this methods add identifier prefixes
        to the Q objects.
        """
        if isinstance(q, Q):
            q.children = [self._update_q_with_prefix(child) for child in q.children]
            return q
        else:
            return (
                LOOKUP_SEP.join((filter_part for filter_part in self.identifiers_prefix + [q[0]] if filter_part)), q[1]
            )

    def get_q(self, value, operator_slug, request):
        if operator_slug not in self.get_allowed_operators():
            raise OperatorFilterError
        else:
            filter_term = self.get_filter_term(self.clean_value(value, operator_slug, request), operator_slug, request)
            return self._update_q_with_prefix(Q(**filter_term) if isinstance(filter_term, dict) else filter_term)

    def clean_value(self, value, operator_slug, request):
        return value

    def get_allowed_operators(self):
        return self.allowed_operators

    def get_filter_term(self, value, operator_slug, request):
        """
        :return: returns Q object or dictionary that will be used for filtering resource response.
        """
        raise NotImplementedError


class SimpleEqualFilterMixin(SimpleFilterMixin):
    """
    Helper that allows filter only with operator equal (=).
    """

    allowed_operators = (OPERATORS.EQ,)


class SimpleFilter(SimpleFilterMixin, Filter):
    """
    Combination of filter and simple filter used for custom resource filters.
    """
    pass


class SimpleMethodFilter(SimpleFilterMixin, MethodFilter):
    """
    Combination of method filter and simple filter used for custom method filters.
    """
    pass


class SimpleModelFieldFilter(SimpleFilterMixin, ModelFieldFilter):
    """
    Combination of model field filter and simple filter used for custom model field filters.
    """
    pass


class SimpleEqualFilter(SimpleEqualFilterMixin, Filter):
    """
    Combination of simple equal filter and simple filter used for custom resource filters.
    """
    pass


class SimpleMethodEqualFilter(SimpleEqualFilterMixin, MethodFilter):
    """
    Combination of simple equal filter and method filter used for custom method filters.
    """
    pass


class SimpleModelFieldEqualFilter(SimpleEqualFilterMixin, ModelFieldFilter):
    """
    Combination of simple equal filter and model field filter used for custom model field filters.
    """
    pass



from distutils.version import StrictVersion

import django
from django.conf.urls import url

from app.resource import (
    IssueResource, UserResource, ExtraResource, CountIssuesPerUserResource, CountWatchersPerIssueResource,
    TestCamelCaseResource, CountWatchersPerIssueResource, UserResource, UserWithFormResource, IssueWithFormResource
)

urlpatterns = [
    url(r'^api/user/$', UserResource.as_view(allowed_methods=('get', 'post', 'head', 'options'))),
    url(r'^api/user-form/$', UserWithFormResource.as_view(allowed_methods=('get', 'post', 'head', 'options'))),
    url(r'^api/user/(?P<pk>\d+)/$', UserResource.as_view(
        allowed_methods=('get', 'put', 'patch', 'delete', 'head', 'options')
    )),
    url(r'^api/test-cc/$', TestCamelCaseResource.as_view(allowed_methods=('get', 'post',))),
    url(r'^api/issue/$', IssueResource.as_view(allowed_methods=('get', 'post', 'head', 'options'))),
    url(r'^api/issue-form/$', IssueWithFormResource.as_view(allowed_methods=('get', 'post', 'head', 'options'))),
    url(r'^api/issue/(?P<pk>\d+)/$',
        IssueResource.as_view(allowed_methods=('get', 'put', 'patch', 'delete', 'head', 'options'))),
    url(r'^api/extra/$', ExtraResource.as_view()),
    url(r'^api/count-issues-per-user/$', CountIssuesPerUserResource.as_view()),
    url(r'^api/count-watchers-per-issue/$', CountWatchersPerIssueResource.as_view()),
]

if StrictVersion(django.get_version()) < StrictVersion('1.9'):
    from django.conf.urls import patterns

    urlpatterns = patterns('', *urlpatterns)



from pyston.serializer import Serializable, SerializableObj

from .models import User


class CountIssuesPerUserTable(Serializable):

    def serialize(self, serialization_format, **kwargs):
        return [
            {
                'email': user.email,
                'created_issues_count': user.created_issues.count(),
            }
            for user in User.objects.all()
        ]


class CountWatchersPerIssue(SerializableObj):

    def __init__(self, issue):
        super(SerializableObj, self).__init__()
        self.name = issue.name
        self.watchers_count = issue.watched_by.count()

    class RESTMeta:
        fields = ('name', 'watchers_count')
